local version = 1
-- モード
local M = {
    Avatar  = 0,  -- 0:Avatar選択
    Proc    = 1,  -- 1:操作選択
    Profile = 2,  -- 2:PROFILE選択
    Parts   = 3,  -- 3:パーツ選択
    Color   = 4,  -- 4:カラー選択
}
W_CUSTOM:SV('Mode', M)

-- アバター一覧
local aList = {}
local avatarList = {}
local selAvatar = W:ReloadParams('Selected') or 1

-- 操作
local procList = {}
local selProc = 1

-- PROFILE
local profileData = {}
local profileList = {}
local selProfile = 1

-- 色・重ね順
local colorData = {}
local colorList = {}
local selColor = 1
local templateColor = {
    color('#ffffffff'),
    color('#c0c0c0'),
    color('#646464ff'),
    color('#000000ff'),
    color('#ff4444ff'),
    color('#ff8044ff'),
    color('#ffff44ff'),
    color('#99ee44ff'),
    color('#22bb55ff'),
    color('#44ddaaff'),
    color('#44ddffff'),
    color('#5577ffff'),
    color('#7744ffff'),
    color('#dd33eeff'),
    color('#800000ff'),
    color('#808000ff'),
    color('#008000ff'),
    color('#000080ff'),
    color('#800080ff'),
    color('#ffeeddff'),
    color('#ffccbbff'),
    color('#aa7755ff'),
    color('#664422ff'),
    color('#332211ff'),
}
W_CUSTOM:SV('TemplateColorList', templateColor)

-- 各パーツの情報
local parts = {}
local partsList = {}
local selParts = 1
local current = {}
local partsData = {}
local currentEmotion
local function IsParts(value)
    return value ~= '<emotion>' and value ~= '<save>'
end
for k,v in pairs({
    {W_CUSTOM:String('Avatar', 'Emotion'), '<emotion>'},
    {'face', 'face1'},
    {'ear', 'ear1'},
    {'back', 'back'},
    {'side', 'side1'},
    {'front', 'front'},
    {'accent', 'accent'},
    {'eye', 'eye2'},
    {'pupil', 'pupil'},
    {'mouth', 'mouth'},
    {'complexion', 'complexion'},
    {'brow', 'brow'},
    {'accessory1', 'accessory1'},
    {'accessory2', 'accessory2'},
    {W_CUSTOM:String('Avatar', 'Exit'), '<save>'},
}) do
    parts[k]= {
        Name = v[1],
        Key  = v[2],
        ColorType = W_AVATAR:Property(v[2], 'ColorType'),
        PartType  = W_AVATAR:Property(v[2], 'PartType'),
    }
    partsList[#partsList+1] = IsParts(v[2]) and W_CUSTOM:String('Avatar', v[1]) or v[1]
end
local emotionList = {'Default', 'Win', 'Lose'}
for _,v in pairs(parts) do
    if v.Key == '<emotion>' then
        partsData[#partsData+1] = {Name = v.Name, Data = emotionList}
        current[#current+1] = 0
    elseif v.Key == '<save>' then
        partsData[#partsData+1] = {Name = v.Name, Data = split(',', W_CUSTOM:String('Save', 'Value'))}
        current[#current+1] = 0
    else
        partsData[#partsData+1] = {Name = v.Name, Data = W_AVATAR:Parts(v.Name)}
        current[#current+1] = 1
    end
end
local saveDescription = {W_CUSTOM:String('Save', 'Description1'), W_CUSTOM:String('Save', 'Description2'), W_CUSTOM:String('Save', 'Description3')}

local screen
local lock = 0.6
local actorSelf
local mode = M.Avatar
local aId
local aData = {}
local isSave = false

local saveFilePath = ''
local saveFileName = ''

-- キー・マウス操作
local function CallbackInput(params)
    local function CheckButton(params, key)
        return (params.button and params.button == key)
            or (params.GameButton and params.GameButton == key)
    end

    -- AVATAR選択 -------------------------------------------------------------------------------------------
    if mode == M.Avatar then
        if params.type == 'InputEventType_Release' then
            if CheckButton(params, 'Back') then
                actorSelf:playcommand('EndCustom')
            end
            if CheckButton(params, 'Start') then
                actorSelf:playcommand('PlayStartSound')
                actorSelf:playcommand('Proc')
            end
        end
        if params.type == 'InputEventType_FirstPress' or params.type == 'InputEventType_Repeat' then
            if CheckButton(params, 'Up') or CheckButton(params, 'Left') then
                selAvatar = selAvatar-1
                if selAvatar <= 0 then
                    selAvatar = #avatarList
                end
                actorSelf:playcommand('SelAvatar')
            end
            if CheckButton(params, 'Down') or CheckButton(params, 'Right') then
                selAvatar = selAvatar+1
                if selAvatar > #avatarList then
                    selAvatar = 1
                end
                actorSelf:playcommand('SelAvatar')
            end
        end
    -- 編集・PROFILEへの紐づけ設定 -------------------------------------------------------------------------------------------
    elseif mode == M.Proc then
        if params.type == 'InputEventType_Release' then
            if CheckButton(params, 'Back') then
                actorSelf:playcommand('PlayCancelSound')
                actorSelf:playcommand('Avatar')
            end
            if CheckButton(params, 'Start') then
                actorSelf:playcommand('PlayStartSound')
                if selProc == 1 then
                    actorSelf:playcommand('Parts')
                else
                    actorSelf:playcommand('Profile')
                end
            end
        end
        if params.type == 'InputEventType_FirstPress' or params.type == 'InputEventType_Repeat' then
            if CheckButton(params, 'Up') or CheckButton(params, 'Left') then
                selProc = selProc-1
                if selProc <= 0 then
                    selProc = #procList
                end
                actorSelf:playcommand('SelProc')
            end
            if CheckButton(params, 'Down') or CheckButton(params, 'Right') then
                selProc = selProc+1
                if selProc > #procList then
                    selProc = 1
                end
                actorSelf:playcommand('SelProc')
            end
        end
    -- PROFILEへの設定・解除 -------------------------------------------------------------------------------------------
    elseif mode == M.Profile then
        if params.type == 'InputEventType_Release' then
            if CheckButton(params, 'Back') then
                actorSelf:playcommand('PlayCancelSound')
                actorSelf:playcommand('Proc')
            end
            if CheckButton(params, 'Start') then
                actorSelf:playcommand('PlayStartSound')
                if selProc == 2 then
                    -- PROFILEに設定
                    W_PLAYER:ProfilePref(profileData[selProfile].Id, 'Avatar', 2, string.gsub(aList[selAvatar].File, W_AVATAR:Path(''), ''))
                    MESSAGEMAN:Broadcast('ShowToast', {Text = string.format(W_CUSTOM:String('Process', 'ApplyText'), profileData[selProfile].Name)})
                else
                    -- PROFILEから解除
                    W_PLAYER:ProfilePref(profileData[selProfile].Id, 'Avatar', 2, '')
                    MESSAGEMAN:Broadcast('ShowToast', {Text = string.format(W_CUSTOM:String('Process', 'ClearText'), profileData[selProfile].Name)})
                end
                actorSelf:playcommand('Proc')
            end
        end
        if params.type == 'InputEventType_FirstPress' or params.type == 'InputEventType_Repeat' then
            if CheckButton(params, 'Up') or CheckButton(params, 'Left') then
                selProfile = selProfile-1
                if selProfile <= 0 then
                    selProfile = #profileList
                end
                actorSelf:playcommand('SelProfile')
            end
            if CheckButton(params, 'Down') or CheckButton(params, 'Right') then
                selProfile = selProfile+1
                if selProfile > #profileList then
                    selProfile = 1
                end
                actorSelf:playcommand('SelProfile')
            end
        end
    -- 色・重ね順の設定・解除 -------------------------------------------------------------------------------------------
    elseif mode == M.Color then
        if params.type == 'InputEventType_Release' then
            if CheckButton(params, 'Back') then
                actorSelf:playcommand('PlayCancelSound')
                if colorData.Type == 'Hair' then
                    actorSelf:playcommand('SetPartsColor', {
                        Color  = colorData.Color,
                        Common = {'HairTopColor'},
                        Target = 'New',
                    })
                    actorSelf:playcommand('SetPartsColor', {
                        Color = colorData.Color2,
                        Common = {'HairBottomColor'},
                        Target = 'New2',
                    })
                elseif colorData.Type == 'Eyes' then
                    actorSelf:playcommand('SetPartsColor', {
                        Color = colorData.Color,
                        Target = 'New',
                    })
                    actorSelf:playcommand('SetPartsColor', {
                        Color = colorData.Color2,
                        Common = {'OddEyeColor'},
                        Target = 'New2',
                    })
                elseif colorData.Type == 'Face' then
                    actorSelf:playcommand('SetPartsColor', {
                        Color  = colorData.Color,
                        Common = {'FaceColor'},
                        Target = 'New',
                    })
                else
                    actorSelf:playcommand('SetPartsColor', {
                        Color = colorData.Color,
                        Order = colorData.Order or nil,
                        Blend = colorData.Blend or nil,
                    })
                end
                actorSelf:playcommand('BackParts')
            end
            if CheckButton(params, 'Start') then
                actorSelf:playcommand('PlayStartSound')
                actorSelf:playcommand('BackParts')
            end
        end
        if params.type == 'InputEventType_FirstPress' or params.type == 'InputEventType_Repeat' then
            local function SelectTemplate(value)
                colorData.SelTemplate = colorData.SelTemplate + value
                if colorData.SelTemplate < 0 then colorData.SelTemplate = #templateColor end
                if colorData.SelTemplate > #templateColor then colorData.SelTemplate = 0 end
                local target = colorData.Both and 'New' or ((colorData.List[selColor] == 'Template') and 'New' or 'New2')
                return {
                    TemplateCursor = colorData.SelTemplate,
                    Color = (colorData.SelTemplate == 0) and colorData[target] or templateColor[colorData.SelTemplate],
                    Target = target,
                }
            end
            local function ColorChange(value)
                local target = 'New'
                for k,v in pairs({
                    R = {Key = 1, Target = 'New'},
                    G = {Key = 2, Target = 'New'},
                    B = {Key = 3, Target = 'New'},
                    R2 = {Key = 1, Target = 'New2'},
                    G2 = {Key = 2, Target = 'New2'},
                    B2 = {Key = 3, Target = 'New2'},
                }) do
                    if colorData.List[selColor] == k then
                        target = colorData.Both and 'New' or v.Target
                        colorData[target][v.Key] = 1.0*(math.floor(colorData[target][v.Key]*100) + value*5) * 0.01
                        if colorData[target][v.Key] < 0 then colorData[target][v.Key] = 0 end
                        if colorData[target][v.Key] > 1 then colorData[target][v.Key] = 1 end
                        break
                    end
                end
                return {Color = colorData[target], Target = target}
            end
            local function OrderChange(value)
                colorData.NewOrder = colorData.NewOrder + value
                if colorData.NewOrder < 1 then colorData.NewOrder = 1 end
                if colorData.NewOrder > 5 then colorData.NewOrder = 5 end
                return {Order = colorData.NewOrder}
            end
            local function BlendChange(...)
                local value = ...
                if value and value ~= '' then
                    colorData.NewBlend = value
                end
                return {Blend = colorData.NewBlend}
            end
            local function ApplyTemplateColor()
                if colorData.List[selColor] == 'Template' and colorData.SelTemplate > 0 then
                    for i=1, 4 do
                        colorData.New[i] = templateColor[colorData.SelTemplate][i]
                    end
                end
                if colorData.List[selColor] == 'Template2' and colorData.SelTemplate > 0 then
                    for i=1, 4 do
                        colorData.New2[i] = templateColor[colorData.SelTemplate][i]
                    end
                end
            end
            if CheckButton(params, 'Up') then
                -- 切り替え前に新色を反映する
                ApplyTemplateColor()
                if colorData.Both and selColor == #colorList then
                    selColor = selColor-5
                else
                    selColor = selColor-1
                end
                if selColor <= 0 then
                    selColor = #colorList
                end
                if string.find(colorData.List[selColor], '^Template.*') then
                    colorData.SelTemplate = 0
                    actorSelf:playcommand('SetPartsColor', SelectTemplate(0))
                elseif colorData.List[selColor] == 'Both' then
                    -- なにもしない
                elseif colorData.List[selColor] == 'Order' then
                    actorSelf:playcommand('SetPartsColor', OrderChange(0))
                elseif colorData.List[selColor] == 'Blend' then
                    actorSelf:playcommand('SetPartsColor', BlendChange())
                else
                    actorSelf:playcommand('SetPartsColor', ColorChange(0))
                end
                actorSelf:playcommand('SelColor')
            end
            if CheckButton(params, 'Down') then
                -- 切り替え前に新色を反映する
                ApplyTemplateColor()
                if colorData.Both and selColor == #colorList - 5 then
                    selColor = #colorList
                else
                    selColor = selColor+1
                end
                if selColor > #colorList then
                    selColor = 1
                end
                if string.find(colorData.List[selColor], '^Template.*') then
                    colorData.SelTemplate = 0
                    actorSelf:playcommand('SetPartsColor', SelectTemplate(0))
                elseif colorData.List[selColor] == 'Both' then
                    -- なにもしない
                elseif colorData.List[selColor] == 'Order' then
                    actorSelf:playcommand('SetPartsColor', OrderChange(0))
                elseif colorData.List[selColor] == 'Blend' then
                    actorSelf:playcommand('SetPartsColor', BlendChange())
                else
                    actorSelf:playcommand('SetPartsColor', ColorChange(0))
                end
                actorSelf:playcommand('SelColor')
            end
            if CheckButton(params, 'Left') then
                actorSelf:playcommand('PlayCursorSound')
                if string.find(colorData.List[selColor], '^Template.*') then
                    actorSelf:playcommand('SetPartsColor', SelectTemplate(-1))
                elseif colorData.List[selColor] == 'Both' then
                    colorData.Both = false
                    actorSelf:playcommand('SetPartsColor', ColorChange(0))
                elseif colorData.List[selColor] == 'Order' then
                    actorSelf:playcommand('SetPartsColor', OrderChange(1))
                elseif colorData.List[selColor] == 'Blend' then
                    actorSelf:playcommand('SetPartsColor', BlendChange('Normal'))
                else
                    actorSelf:playcommand('SetPartsColor', ColorChange(-1))
                end
            end
            if CheckButton(params, 'Right') then
                actorSelf:playcommand('PlayCursorSound')
                if string.find(colorData.List[selColor], '^Template.*') then
                    actorSelf:playcommand('SetPartsColor', SelectTemplate(1))
                elseif colorData.List[selColor] == 'Both' then
                    colorData.Both = true
                    for i=1, 4 do colorData.New2[i] = colorData.New[i] end
                    actorSelf:playcommand('SetPartsColor', ColorChange(0))
                elseif colorData.List[selColor] == 'Order' then
                    actorSelf:playcommand('SetPartsColor', OrderChange(-1))
                elseif colorData.List[selColor] == 'Blend' then
                    actorSelf:playcommand('SetPartsColor', BlendChange('Add'))
                else
                    actorSelf:playcommand('SetPartsColor', ColorChange(1))
                end
            end
        end
    -- AVATARパーツ選択 -------------------------------------------------------------------------------------------
    elseif mode == M.Parts then
        if params.type == 'InputEventType_Release' then
            if CheckButton(params, 'Back') then
                if selParts ~= #partsData then
                    -- 戻るボタンを押したときは戻らずに保存にカーソルを持っていくだけ
                    selParts = #partsData
                else
                    -- ただし、すでに保存にカーソルがあっている場合は終了に合わせる
                    current[selParts] = 2
                end
                actorSelf:playcommand('SelParts')
            end
            if CheckButton(params, 'Start') then
                if selParts == #partsData then
                    if current[selParts] == 0 then
                        -- 新規保存
                        actorSelf:playcommand('PlayStartSound')
                        actorSelf:playcommand('SaveAvatar')
                    elseif current[selParts] == 1 then
                        -- 上書き保存
                        actorSelf:playcommand('PlayStartSound')
                        if aList[selAvatar].Type == 'User' then
                            actorSelf:playcommand('SaveAvatarIni', {Path = saveFilePath, Name = saveFileName})
                        elseif aList[selAvatar].Type == 'New' then
                            actorSelf:playcommand('SaveAvatar')
                        else
                            MESSAGEMAN:Broadcast('ShowToast', {Text = W_CUSTOM:String('Save', 'DenySample')})
                            actorSelf:playcommand('SaveAvatar')
                        end
                    else
                        actorSelf:playcommand('PlayCancelSound')
                        -- これをいれないとOF AlphaVでディレクトリ内のファイル一覧が更新されない
                        if W:IsOFV() then
                            THEME:ReloadMetrics()
                        end
                        actorSelf:playcommand('Avatar')
                    end
                else
                    -- 色・重ね順を変更
                    local colorType = W_AVATAR:Property(parts[selParts].Key, 'ColorType')
                    if colorType ~= 'Fixed' and parts[selParts].Key ~= '<emotion>' then
                        actorSelf:playcommand('PlayStartSound')
                        actorSelf:playcommand('Color', {
                            ColorType   = colorType,
                            IsAccessory = string.find(parts[selParts].Name, '^accessory.+') ~= nil,
                            IsEyes      = parts[selParts].Name == 'eye',
                        })
                    end
                end
            end
        end
        if params.type == 'InputEventType_FirstPress' or params.type == 'InputEventType_Repeat' then
            if CheckButton(params, 'Up') then
                selParts = selParts-1
                if selParts <= 0 then
                    selParts = #partsData
                end
                actorSelf:playcommand('SelParts')
            end
            if CheckButton(params, 'Down') then
                selParts = selParts+1
                if selParts > #partsData then
                    selParts = 1
                end
                actorSelf:playcommand('SelParts')
            end
            if CheckButton(params, 'Left') then
                actorSelf:playcommand('PlayCursorSound')
                current[selParts] = current[selParts]-1
                if current[selParts] < 0 then
                    current[selParts] = #partsData[selParts].Data-1
                end
                -- 表情変更の場合は各パーツのカーソルを変更する
                if selParts == 1 then
                    currentEmotion = partsData[1].Data[current[1]+1]
                    actorSelf:playcommand('SetPartsCursor')
                end
                actorSelf:playcommand('ChangeParts')
            end
            if CheckButton(params, 'Right') then
                actorSelf:playcommand('PlayCursorSound')
                current[selParts] = current[selParts]+1
                if current[selParts] > #partsData[selParts].Data-1 then
                    current[selParts] = 0
                end
                -- 表情変更の場合は各パーツのカーソルを変更する
                if selParts == 1 then
                    currentEmotion = partsData[1].Data[current[1]+1]
                    actorSelf:playcommand('SetPartsCursor')
                end
                actorSelf:playcommand('ChangeParts')
            end
        end
    end
end

return Def.ActorFrame({
    InitCommand = function(self)
        actorSelf = self
    end,
    OnCommand = function(self)
        screen = SCREENMAN:GetTopScreen()
        screen:lockinput(lock)
        screen:AddInputCallback(CallbackInput)
        MESSAGEMAN:Broadcast('UpdateHelper', {Text = W_CUSTOM:String('Helper', 'Profile')})
        self:playcommand('Avatar')
    end,
    -- Avatar選択に遷移
    AvatarCommand = function(self)
        mode = M.Avatar
        -- アバターデータの更新
        aList = {{Type = 'New', File = '<default>', Name = W_CUSTOM:String('Avatar', 'New')}}
        -- ユーザー作成のデータを取得
        local fileList = FILEMAN:GetDirListing(W_AVATAR:Path('user/'))
        for _,v in pairs(fileList) do
            local path = W_AVATAR:Path('user/'..v)
            if ActorUtil.GetFileType(path) == 'FileType_Ini' then
                local file = IniFile.ReadFile(path)
                aList[#aList+1] = {Type = 'User', File = path, Name = file.main and file.main.Name or file.Name or 'No Name'}
            end
        end
        fileList = nil
        -- テンプレート
        for _,d in pairs(FILEMAN:GetDirListing(W_AVATAR:Path('/')), true) do
            if d ~= 'user' then
                for _,v in pairs(FILEMAN:GetDirListing(W_AVATAR:Path(d..'/template/'))) do
                    local path = W_AVATAR:Path(d..'/template/'..v)
                    if ActorUtil.GetFileType(path) == 'FileType_Ini' then
                        local file = IniFile.ReadFile(path)
                        aList[#aList+1] = {Type = 'Template', File = path, Name = file.main and file.main.Name or file.Name or 'No Name'}
                    end
                end
            end
        end
        avatarList = {}
        for _,v in pairs(aList) do
            avatarList[#avatarList+1] = ((v.Type == 'Template') and '* ' or '')..v.Name
        end
        MESSAGEMAN:Broadcast('ChangeMode', {Mode = mode})
        MESSAGEMAN:Broadcast('UpdateScrollData', {Data = avatarList})
        MESSAGEMAN:Broadcast('ScrollMenu', {Cursor = selAvatar, Init = true})
        self:playcommand('SetAvatarText')
    end,
    -- Avatarを選択
    SelAvatarCommand = function(self)
        MESSAGEMAN:Broadcast('ScrollMenu', {Cursor = selAvatar})
        MESSAGEMAN:Broadcast('ChangeAvatarFile', {Id = 'avatar', File = aList[selAvatar].File})
        self:playcommand('SetAvatarText')
    end,
    SetAvatarTextCommand = function(self)
        MESSAGEMAN:Broadcast('ChangeTitle', {Text = aList[selAvatar].Name, Type = aList[selAvatar].Type})
        local fileName = split('/', aList[selAvatar].File)
        MESSAGEMAN:Broadcast('SetExtraText', {
            Text1 = W_CUSTOM:String('Avatar', 'Type'), Text2 = W_CUSTOM:String('Avatar', aList[selAvatar].Type),
            Text3 = W_CUSTOM:String('Avatar', 'Filename'), Text4 = (fileName[1] == '<default>') and '---' or fileName[#fileName],
        })
    end,
    -- 処理選択に遷移
    ProcCommand = function(self)
        mode = M.Proc
        MESSAGEMAN:Broadcast('ChangeMode', {Mode = mode})
        procList = {}
        if aList[selAvatar].Type == 'User' then
            procList[#procList+1] = W_CUSTOM:String('Process', 'Edit')
        elseif aList[selAvatar].Type == 'Template' then
            procList[#procList+1] = W_CUSTOM:String('Process', 'CopyAndEdit')
        else
            mode = M.Parts
        end
        if mode == M.Proc then
            -- このタイミングで最新のPROFILE一覧を取得
            profileData = W_PLAYER:List()
            if #profileData > 0 then
                procList[#procList+1] = W_CUSTOM:String('Process', 'ApplyLabel')
                procList[#procList+1] = W_CUSTOM:String('Process', 'ClearLabel')
            end
            MESSAGEMAN:Broadcast('UpdateScrollData', {Data = procList})
            MESSAGEMAN:Broadcast('ScrollMenu', {Cursor = selProc, Init = true})
            MESSAGEMAN:Broadcast('SetExtraText', {
                Text1 = W_CUSTOM:String('Process', 'Name'), Text2 = aList[selAvatar].Name,
                Text3 = W_CUSTOM:String('Avatar', 'Type'), Text4 = W_CUSTOM:String('Avatar', aList[selAvatar].Type),
            })
        else
            self:playcommand('Parts')
        end
    end,
    -- 処理を選択
    SelProcCommand = function(self)
        MESSAGEMAN:Broadcast('ScrollMenu', {Cursor = selProc})
    end,
    -- PROFILE選択に遷移
    ProfileCommand = function(self)
        mode = M.Profile
        MESSAGEMAN:Broadcast('ChangeMode', {Mode = mode})
        profileList = {}
        for _,v in pairs(profileData) do
            profileList[#profileList+1] = v.Name
        end
        selProfile = 1
        MESSAGEMAN:Broadcast('UpdateScrollData', {Data = profileList})
        MESSAGEMAN:Broadcast('ScrollMenu', {Cursor = selProfile, Init = true})
        self:playcommand('SetProfileText')
    end,
    -- PROFILE選択
    SelProfileCommand = function(self)
        MESSAGEMAN:Broadcast('ScrollMenu', {Cursor = selProfile})
        self:playcommand('SetProfileText')
    end,
    SetProfileTextCommand = function(self)
        local current = split('/', W_PLAYER:ProfilePref(profileData[selProfile].Id, 'Avatar', 1) or '')
        MESSAGEMAN:Broadcast('SetExtraText', {
            Text1 = W_CUSTOM:String('Profile', 'Name'),
            Text2 = profileList[selProfile],
            Text3 = W_CUSTOM:String('Profile', 'Current'),
            Text4 = (current[1] == '') and W_CUSTOM:String('Profile', 'None') or current[#current],
        })
    end,
    -- 色・重ね順選択に遷移
    ColorCommand = function(self, params)
        mode = M.Color
        MESSAGEMAN:Broadcast('ChangeMode', {Mode = mode})
        local part = parts[selParts].Name
        selColor = 1
        colorData = {
            Type = params.ColorType,
            SelTemplate = 0,
        }
        if params.IsEyes then
            local leftColor  = aData[part].Color or Color('White')
            local rightColor = aData.common.OddEyeColor or aData[part].Color or Color('White')
            colorList = split(',', W_CUSTOM:String('ColorList', 'Eyes'))
            colorData.Type = 'Eyes'
            colorData.Both  = (join(',', leftColor) == join(',', rightColor))
            colorData.List = {'Template', 'R', 'G', 'B', 'Template2', 'R2', 'G2', 'B2', 'Both'}
            colorData.Color = {}
            colorData.New   = {}
            colorData.Color2 = {}
            colorData.New2   = {}
            for i=1, 4 do
                colorData.Color[i] = leftColor[i]
                colorData.New[i]   = leftColor[i]
                colorData.Color2[i] = rightColor[i]
                colorData.New2[i]   = rightColor[i]
            end
            MESSAGEMAN:Broadcast('SetBothColorText', {
                Text = W_CUSTOM:String('Both', 'Eyes'),
            })
            MESSAGEMAN:Broadcast('SetBothColorCursor', {Cursor = colorData.Both and 1 or 0})
        elseif params.ColorType == 'Hair' then
            local curTopColor = aData.common.HairTopColor or aData[part].Color or Color('White')
            local curBottomColor = aData.common.HairBottomColor or aData[part].Color or Color('White')
            colorList = split(',', W_CUSTOM:String('ColorList', 'Hair'))
            colorData.Both  = (join(',', curTopColor) == join(',', curBottomColor))
            colorData.List = {'Template', 'R', 'G', 'B', 'Template2', 'R2', 'G2', 'B2', 'Both'}
            colorData.Color = {}
            colorData.New   = {}
            colorData.Color2 = {}
            colorData.New2   = {}
            for i=1, 4 do
                colorData.Color[i] = curTopColor[i]
                colorData.New[i]   = curTopColor[i]
                colorData.Color2[i] = curBottomColor[i]
                colorData.New2[i]   = curBottomColor[i]
            end
            MESSAGEMAN:Broadcast('SetBothColorText', {
                Text = W_CUSTOM:String('Both', 'Hair'),
            })
            MESSAGEMAN:Broadcast('SetBothColorCursor', {Cursor = colorData.Both and 1 or 0})
        elseif params.ColorType == 'Face' then
            local curColor = aData.common.FaceColor or Color('White')
            colorList = split(',', W_CUSTOM:String('ColorList', 'Default'))
            colorData.List = {'Template', 'R', 'G', 'B'}
            colorData.Color = {}
            colorData.New   = {}
            for i=1, 4 do
                colorData.Color[i] = curColor[i]
                colorData.New[i]   = curColor[i]
            end
        else
            local curColor = aData[part].Color or Color('White')
            colorList = split(',', W_CUSTOM:String('ColorList', 'Default'))
            colorData.List = {'Template', 'R', 'G', 'B'}
            colorData.Color = {}
            colorData.New   = {}
            for i=1, 4 do
                colorData.Color[i] = curColor[i]
                colorData.New[i]   = curColor[i]
            end
            if params.IsAccessory then
                colorList[#colorList+1]           = W_CUSTOM:String('ColorList', 'Order')
                colorData.List[#colorData.List+1] = 'Order'
                colorData.Order    = aData[part].Order or 5
                colorData.NewOrder = aData[part].Order or 5
                colorList[#colorList+1]           = W_CUSTOM:String('ColorList', 'Blend')
                colorData.List[#colorData.List+1] = 'Blend'
                colorData.Blend    = aData[part].Blend or 'Normal'
                colorData.NewBlend = aData[part].Blend or 'Normal'
                MESSAGEMAN:Broadcast('SetBothColorCursor', {Cursor = (colorData.Blend == 'Normal') and 0 or 1})
            end
        end

        MESSAGEMAN:Broadcast('SetExtraText', {Text1 = '', Text2 = '',Text3 = '', Text4 = '',})
        MESSAGEMAN:Broadcast('UpdateScrollData', {Data = colorList})
        MESSAGEMAN:Broadcast('ScrollMenu', {Cursor = selColor, Init = true})
        MESSAGEMAN:Broadcast('SetColorListCursor', {Cursor = 0})
        self:playcommand('SelColorControll')
    end,
    -- 色・重ね順選択
    SelColorCommand = function(self)
        MESSAGEMAN:Broadcast('ScrollMenu', {Cursor = selColor})
        self:playcommand('SelColorControll')
    end,
    -- 縦方向移動時の設定パネル（アバター画像の横の項目）の内容切り替え
    SelColorControllCommand = function(self, params)
        local isTemplate = string.find(colorData.List[selColor], '^Template.*') ~= nil
        if isTemplate then
            MESSAGEMAN:Broadcast('ShowColorList')
        else
            MESSAGEMAN:Broadcast('HideColorList')
        end
        if not isTemplate and (colorData.List[selColor] == 'Both' or colorData.List[selColor] == 'Order' or colorData.List[selColor] == 'Blend') then
            if colorData.List[selColor] == 'Order' then
                MESSAGEMAN:Broadcast('SetBothColorText', {
                    Text = W_CUSTOM:String('Order', 'Accessory'),
                })
            elseif colorData.List[selColor] == 'Blend' then
                MESSAGEMAN:Broadcast('SetBothColorText', {
                    Text = W_CUSTOM:String('Blend', 'Accessory'),
                })
            end
            MESSAGEMAN:Broadcast('ShowBothColor', {Type = colorData.List[selColor]})
        else
            MESSAGEMAN:Broadcast('HideBothColor')
        end
        if not isTemplate and colorData.List[selColor] ~= 'Both' and colorData.List[selColor] ~= 'Order' and colorData.List[selColor] ~= 'Blend' then
            MESSAGEMAN:Broadcast('ShowRGBColor')
        else
            MESSAGEMAN:Broadcast('HideRGBColor')
        end
    end,
    -- 設定パネル操作時の処理
    SetPartsColorCommand = function(self, params)
        local part = parts[selParts].Name
        local common = nil
        if colorData.Type == 'Hair' then
            MESSAGEMAN:Broadcast('SetBothColorCursor', {Cursor = colorData.Both and 1 or 0})
            if colorData.Both then
                common = {'HairTopColor', 'HairBottomColor'}
            else
                common = {(params.Target and params.Target == 'New') and 'HairTopColor' or 'HairBottomColor'}
            end
        elseif colorData.Type == 'Face' then
            common = {'FaceColor'}
        elseif colorData.Type == 'Eyes' then
            MESSAGEMAN:Broadcast('SetBothColorCursor', {Cursor = colorData.Both and 1 or 0})
            if colorData.Both or params.Target == 'New2' then
                common = {'OddEyeColor'}
            end
        elseif colorData.List[selColor] == 'Order' then
            aData[part].Order = params.Order
            MESSAGEMAN:Broadcast('SetBothColorCursor', {Cursor = 5 - (params.Order or 5)})
        elseif colorData.List[selColor] == 'Blend' then
            aData[part].Blend = params.Blend
            MESSAGEMAN:Broadcast('SetBothColorCursor', {Cursor = (params.Blend == 'Normal') and 0 or 1})
        end
        if params.TemplateCursor then
            MESSAGEMAN:Broadcast('SetColorListCursor', {Cursor = params.TemplateCursor})
        else
            MESSAGEMAN:Broadcast('SetRGBColorCursor', {
                Color = colorData[params.Target or 'New'],
            })
        end
        if common then
            for _,v in pairs(common) do
                MESSAGEMAN:Broadcast('ChangeAvatarParts', {
                    Id = aId,
                    common = {[v] = '#'..ColorToHex(params.Color)},
                })
            end
            if colorData.Type == 'Eyes' and not colorData.Both and params.Target == 'New2' then return end
        end
        MESSAGEMAN:Broadcast('ChangeAvatarParts', {
            Id = aId,
            [part] = {
                Color = params.Color and '#'..ColorToHex(params.Color) or nil,
                Order = params.Order or nil,
                Blend = params.Blend or nil,
            },
        })
    end,
    -- パーツ
    PartsCommand = function(self)
        mode = M.Parts
        MESSAGEMAN:Broadcast('ChangeMode', {Mode = mode})
        selParts = 1
        currentEmotion = 'Default'
        -- セーブの初期値
        current[#current] = (aList[selAvatar].Type == 'User') and 1 or 0
        saveFilePath = aList[selAvatar].File
        saveFileName = aList[selAvatar].Name
        MESSAGEMAN:Broadcast('UpdateScrollData', {Data = partsList})
        MESSAGEMAN:Broadcast('ScrollMenu', {Cursor = selParts, Init = true})
        MESSAGEMAN:Broadcast('SetAvatarEmotion', {Id = aId, Emotion = 'Default'})
        self:playcommand('SetPartsText')
        -- アバターデータの取得(aDataに格納)
        MESSAGEMAN:Broadcast('GetSaveData')
        MESSAGEMAN:Broadcast('SetExtraText', {
            Text1 = W_CUSTOM:String('Avatar', 'GroupName'), Text2 = '---',
            Text3 = W_CUSTOM:String('Avatar', 'PartName'),  Text4 = '---',
        })
    end,
    -- 色・重ね順設定から戻った
    BackPartsCommand = function(self)
        MESSAGEMAN:Broadcast('HideRGBColor')
        MESSAGEMAN:Broadcast('HideColorList')
        MESSAGEMAN:Broadcast('HideBothColor')
        mode = M.Parts
        MESSAGEMAN:Broadcast('ChangeMode', {Mode = mode})
        MESSAGEMAN:Broadcast('UpdateScrollData', {Data = partsList})
        MESSAGEMAN:Broadcast('ScrollMenu', {Cursor = selParts, Init = true})
        self:playcommand('SetPartsText')
    end,
    SelPartsCommand = function(self)
        MESSAGEMAN:Broadcast('ScrollMenu', {Cursor = selParts})
        self:playcommand('SetPartsText')
    end,
    -- 表示テキストの更新
    SetPartsTextCommand = function(self)
        local key  = parts[selParts].Key
        local name = parts[selParts].Name
        if key == '<emotion>' then
            MESSAGEMAN:Broadcast('ChangeTitle', {Text = W_CUSTOM:String('Emotion', emotionList[current[selParts] + 1])})
        elseif key == '<save>' then
            local sel = current[selParts] + 1
            MESSAGEMAN:Broadcast('ChangeTitle', {Text = partsData[selParts].Data[sel]..'\n'..saveDescription[sel]})
        else
            MESSAGEMAN:Broadcast('ChangeTitle', {Text = W_CUSTOM:String('Avatar', parts[selParts].Name)})
        end
        if IsParts(key) then
            MESSAGEMAN:Broadcast('SetExtraText', {
                Text1 = W_CUSTOM:String('Avatar', 'GroupName'), Text2 = aData[name][currentEmotion].Group,
                Text3 = W_CUSTOM:String('Avatar', 'PartName'),  Text4 = aData[name][currentEmotion].File,
            })
        else
            MESSAGEMAN:Broadcast('SetExtraText', {
                Text1 = W_CUSTOM:String('Avatar', 'GroupName'), Text2 = '---',
                Text3 = W_CUSTOM:String('Avatar', 'PartName'),  Text4 = '---',
            })
        end
    end,
    -- 現在の選択位置を変更（初期遷移時と表情切り替え時に実行）
    -- partsData[#partsData+1] = {Name = v.Name, Data = W_AVATAR:Parts(v.Name)}
    SetPartsCursorCommand = function(self)
        for k,v in pairs(parts) do
            if IsParts(v.Key) then
                -- ヒットしない場合は一番端
                current[k] = 0
                -- 各パーツのGroup、ファイル名を現在のAvatarの設定と比較
                for cursor,data in pairs(partsData[k].Data or {}) do
                    --if v.Name == 'brow' then _SYS2({currentEmotion,data, aData[v.Name]}) end
                    if data.Group == aData[v.Name][currentEmotion].Group and data.Name == aData[v.Name][currentEmotion].File then
                        current[k] = cursor-1
                        break
                    end
                end
            end
        end
    end,
    -- パーツを選択
    ChangePartsCommand = function(self)
        local name = parts[selParts].Name
        local key  = parts[selParts].Key
        if key == '<emotion>' or key == '<save>' then
            self:playcommand('SetPartsText')
            MESSAGEMAN:Broadcast('SetExtraText', {
                Text1 = W_CUSTOM:String('Avatar', 'GroupName'), Text2 = '---',
                Text3 = W_CUSTOM:String('Avatar', 'PartName'),  Text4 = '---',
            })
        else
            local currentParts = partsData[selParts].Data[current[selParts]+1] or nil
            local group = currentParts and currentParts.Group or 'waiei'
            local file  = currentParts and currentParts.Name  or '00'
            MESSAGEMAN:Broadcast('ChangeAvatarParts', {
                Id = aId,
                [name] = {
                    [currentEmotion] = {Group = group,File  = file,}
                },
            })
            MESSAGEMAN:Broadcast('SetExtraText', {
                Text1 = W_CUSTOM:String('Avatar', 'GroupName'), Text2 = group,
                Text3 = W_CUSTOM:String('Avatar', 'PartName'),  Text4 = file,
            })
        end
        MESSAGEMAN:Broadcast('ChangeEmotion', {Emotion = currentEmotion})
    end,
    -- 保存名設定
    SaveAvatarCommand = function(self)
        SCREENMAN:AddNewScreenToTop("ScreenTextEntry")
        local errMsg = W_CUSTOM:String('Save', 'InvalidCharacter')
        local settings = {
            Question = W_CUSTOM:String('Save', 'SaveName'),
            InitialAnswer = '',
            MaxInputLength = 20,
            Validate = function(answer, errOut)
                return string.find(answer, '^[A-z0-9_ ]+$') ~= nil, errMsg
            end,
            OnOK = function(name)
                SCREENMAN:GetTopScreen():Cancel()
                self:playcommand('SaveAvatarIni', {Path = 'user/'..name..'.ini', Name = name})
                screen:lockinput(lock)
            end,
            OnCancel = function()
                screen:lockinput(lock)
                self:playcommand('CancelName')
            end,
        };
        SCREENMAN:GetTopScreen():Load(settings)
    end,
    -- 保存処理
    SaveAvatarIniCommand = function(self, params)
        -- 保存時に付与するのですでにアバターPathが含まれている場合はいったん削除する
        saveFilePath = string.gsub(params.Path, W_AVATAR:Path(''), '')
        saveFileName = params.Name
        isSave = true
        MESSAGEMAN:Broadcast('GetSaveData')
    end,
    -- データ取得
    GetAvatarDataMessageCommand = function(self, params)
        aData = params.Data
        aId = params.Id
        if not isSave then
            self:playcommand('SetPartsCursor')
            return
        end
        local saveData = {}
        saveData.main = {
            Name = saveFileName,
        }
        saveData.citmin = {
            Version = W:Ver(),
        }
        saveData.common = {
            Version = version,
        }
        for _,v in pairs({'HairTopColor', 'HairBottomColor', 'FaceColor', 'OddEyeColor'}) do
            saveData.common[v] = aData.common[v] and '#'..ColorToHex(aData.common[v]) or nil
        end
        for _,v in pairs(parts) do
            if IsParts(v.Key) then
                local data = aData[v.Name]
                local emotionMode = W_AVATAR:Property(v.Key, 'EmotionMode')
                local colorType   = W_AVATAR:Property(v.Key, 'ColorType')
                saveData[v.Name] = {}
                saveData[v.Name].Default = (data.Default.Group and data.Default.File) and data.Default.Group..'/'..data.Default.File or nil
                -- 表情が固定ではない
                if #emotionMode ~= 1 then
                    saveData[v.Name].Win  = (data.Win.Group  and data.Win.File)  and data.Win.Group..'/'..data.Win.File or nil
                    saveData[v.Name].Lose = (data.Lose.Group and data.Lose.File) and data.Lose.Group..'/'..data.Lose.File or nil
                end
                -- 色変更が可能
                if colorType == 'Normal' then
                    saveData[v.Name].Color = data.Color and '#'..ColorToHex(data.Color) or nil
                end
                -- 重ね順序の指定が可能
                if string.find(v.Name, '^accessory.*') then
                    saveData[v.Name].Order = data.Order or nil
                    saveData[v.Name].Blend = data.Blend or nil
                end
            end
        end
        -- 新規保存で同じファイル名が存在する場合は数字を後ろにつける
        if current[selParts] == 0 and FILEMAN:DoesFileExist(W_AVATAR:Path(saveFilePath)) then
            local count = 1
            local fileWithoutPrefix = string.gsub(saveFilePath, '.ini', '')
            while(FILEMAN:DoesFileExist(W_AVATAR:Path(fileWithoutPrefix..string.format('_%02d.ini', count)))) do
                count = count + 1
            end
            saveFilePath = fileWithoutPrefix..string.format('_%02d.ini', count)
        end
        IniFile.WriteFile(W_AVATAR:Path(saveFilePath), saveData)

        MESSAGEMAN:Broadcast('ShowToast', {Text = string.format(W_CUSTOM:String('Save', 'Complete'), W_AVATAR:Path(saveFilePath))})
        -- 連続で新規作成しないように上書きにカーソルを移動
        current[selParts] = 1
        aList[selAvatar] = {Type = 'User', File = saveFilePath, Name = saveFileName}
        isSave = false
        self:playcommand('SetPartsText')
        self:playcommand('CancelName')
    end,
    Def.Quad({
        InitCommand = function(self)
            self:zoomto(SCREEN_WIDTH, 260)
            self:Center()
            self:fadeleft(0.25)
            self:faderight(0.25)
            self:diffuse(0, 0, 0, 0.75)
            self:visible(false)
        end,
        SaveAvatarCommand = function(self)
            self:visible(true)
        end,
        CancelNameCommand = function(self)
            self:visible(false)
        end,
    }),
    W_CUSTOM:Template('Basic'),
})