local params = ...
local size  = (params and params.Size) and params.Size or 256   -- 基準となるサイズ（横幅）
local ratio = (params and params.Ratio) and params.Ratio or 1.0 -- 最大縦幅時の縦横比
local cache = (params and params.Cache) and params.Cache or 0.0 -- キャッシュ表示時間(0はキャッシュ無し、0未満は常にキャッシュ)
local rate  = (params and params.Rate) and params.Rate or 1.0   -- 動画バナーの再生速度（複数表示時に高速表示される対策）
local mode = (params and params.Mode) and params.Mode or 'Banner'   -- バナーかジャケットか

if W:IsOF() then
    rate = 1.0
end

local fallback = THEME:GetPathG('Common fallback', string.lower(mode))

-- 設定があれば更新
local function UpdateParams(params)
    if params then
        size  = params.Size and params.Size or size
        ratio = params.Ratio and params.Ratio or ratio
        cache = params.Cache and params.Cache or cache
        rate  = params.Rate and params.Rate or rate
        if W:IsOF() then
            rate = 1.0
        end
    end
end

local width  = 0
local height = 0
-- 画像のサイズ情報を取得
-- 画像をロードしてから呼び出すこと
local function GetLoadFileDetail(self)
    local maxWidth  = size
    local maxHeight = 1.0 * size / ratio
    local cropSize  = (1.0 - maxHeight / (height * size / width)) / 2
    local realHeight = math.min(maxHeight, height * size / width)
    return {
        Width      = width,
        Height     = height,
        MaxWidth   = maxWidth,
        MaxHeight  = maxHeight,
        RealHeight = realHeight,
        CropSize   = cropSize,
    }
end

-- Actorを返却
local function Actor(self)
    -- バナーパス
    local path = ''

    -- コマンドまとめ
    local function CropCommand(self)
        local d = GetLoadFileDetail(self)
        self:position(0)
        -- 縦に長いバナーは切り抜く （横に長い場合は縮小）
        if d.Height/d.Width < d.MaxHeight/d.MaxWidth then
            self:scaletofit(-d.MaxWidth/2, -d.MaxHeight/2, d.MaxWidth/2, d.MaxHeight/2)
        else
            self:scaletocover(-d.MaxWidth/2, -d.MaxHeight/2, d.MaxWidth/2, d.MaxHeight/2)
        end
        self:croptop(d.CropSize)
        self:cropbottom(d.CropSize)
    end

    -- バナー画像情報が更新されたことを通知
    local function CallUpdatedMessage(self)
        local d = GetLoadFileDetail(self)
        self:GetParent():playcommand('CallbackBanner', d)
    end

    return Def.Banner({
        InitCommand = function(self)
            self:visible(false)
        end,
        -- キャッシュから実際の画像に置き換え
        DrawRealCommand = function(self)
            self:finishtweening()
            self:LoadBanner(path)
            self:rate(rate)
            width  = self:GetWidth()
            height = self:GetHeight()
            CropCommand(self)
        end,
        -- バナー情報が更新された
        UpdateBannerCommand = function(self, params)
            UpdateParams(params)
            path = params.Path or fallback
            if path == '' then path = fallback end
            -- FallbackまたはジャケモードでOF以外の場合はキャッシュを使わない
            local currentCache = cache
            if path == fallback or (mode == 'Jacket' and not W:IsOF()) then
                currentCache = 0
            end
            self:stoptweening()
            if currentCache ~= 0 then
                if W:IsOF() then
                    self:LoadFromCached(mode, path)
                else
                    self:LoadFromCachedBanner(path)
                end
                self:rate(0)
            else
                self:LoadBanner(path)
                self:rate(rate)
            end
            width  = self:GetWidth()
            height = self:GetHeight()
            CallUpdatedMessage(self)
            CropCommand(self)
            self:visible(true)
            -- キャッシュ表示時間が0より大きい場合、sleep後実際の画像を表示
            if currentCache > 0 then
                self:sleep(currentCache)
                self:queuecommand('DrawReal')
            end
        end,
    })
end

return {
    Detail = GetLoadFileDetail,
	Actor  = Actor,
}