local params  = ...
local graphW  = params and params.Width or SCREEN_WIDTH
local max     = params and params.Max or 20
local height  = params and params.Size or SCREEN_HEIGHT / max
local bgColor = params and params.BgColor or color('0, 0, 0, 0.66')
local bgFade  = (params and params.BgFade == nil) and true or params.BgFade
local pn      = params and params.Player
local normalizeAlpha = (1.0 - bgColor[4]) * 0.8

local graphH = max
-- 1メモリの秒数計算用 ceil(楽曲秒数/この値)が1メモリ分の秒数
local memUnit = 120.0
--[[
    TODO:
        ・前のメモリ位置との平均を表示させてもう少し滑らかなグラフに
        ・DWIは…いらないか
]]
local function UpdateGraph(params)
    local song = params.Song
    if not song then return end
    local path = song:GetSongFilePath()
    local extension = string.find(path, '.+%.ssc') and 'ssc' or (string.find(path, '.+%.sm') and 'sm' or nil)
    if not extension then return end
    local steps      = W:Steps(pn)
    local timing     = steps:GetTimingData()
    local stepType   = string.upper(string.gsub(ToEnumShortString(steps:GetStepsType()), '_', '-'))
    local difficulty = string.upper(ToEnumShortString(steps:GetDifficulty()))
    local warps      = {}
    for _,v in pairs(timing:GetWarps() or {}) do
        local splitWarp = split('=', v)
        warps[#warps+1] = {tonumber(splitWarp[1]), tonumber(splitWarp[2])}
    end
    local warpIndex = 1
    --local stepStartSec = params.FirstSec
    -- positionバーは0秒から開始
    local songLenSec = (params.LastSec - 0)

    local stepsPerSecList = {0}
    local file = RageFileUtil:CreateRageFile()
    file:Open(path, 1)
    file:Seek(0)
    -- Steps取得のためのチェック用変数
    local hasStepType   = false
    local hasDifficulty = false
    local discription = ''
    local line = ''         -- 1行分のデータ
    local lineData = {}     -- 1行分のデータを1小節分まとめる
    local currentVertex = 1     -- 現在の頂点カウント
    local currentBeat = 0
    local currentSec = timing:GetElapsedTimeFromBeat(currentBeat)
    local nextSec = 0
    local secUnit = math.ceil(songLenSec / memUnit)  -- 1メモリの秒数
    nextSec = secUnit
    -- 譜面情報の読み取り
    local RNM = {HEADER = 1, PROPERTIES = 2, NOTE = 3}
    local readNoteMode = RNM.HEADER
    while true do
        line = string.upper(file:GetLine())
        if readNoteMode == RNM.PROPERTIES then
            -- ノート先頭の譜面情報部分であれば飛ばす
            if not string.find(line, '^.+:') then
                readNoteMode = RNM.NOTE
            end
        end
        if readNoteMode == RNM.NOTE then
            -- 小節の区切り、ノートの終了
            if string.find(line, '[,;].*') then
                -- 1行の拍数
                local oneBeat = 4.0 / #lineData
                --currentBeat = currentBar * 4.0
                -- 1行ずつチェック
                for _,v in pairs(lineData) do
                    -- ワープ内の矢印は無視
                    local isWarp = false
                    local latestWarp = -1
                    for i=warpIndex, #warps do
                        if warps[i][1] <= currentBeat
                            and currentBeat < warps[i][1] + warps[i][2]
                            and warps[i][1] ~= latestWarp then
                            isWarp = true
                            break
                        end
                        if warps[i][1] > currentBeat then
                            break
                        end
                        warpIndex = i
                        latestWarp = warps[i][1] + warps[i][2]
                    end
                    currentSec = timing:GetElapsedTimeFromBeat(currentBeat)
                    -- positionバーは0秒から開始なのでそれより前のbeatは無視する
                    if currentSec >= 0 then
                        -- 1メモリ分の時間を超えていたら次の配列番号へ
                        if params.LastSec >= nextSec and currentSec >= nextSec then
                            while currentVertex*secUnit < currentSec do
                                currentVertex = currentVertex + 1
                                nextSec = nextSec + secUnit
                                stepsPerSecList[currentVertex] = 0
                            end
                        end
                        -- 現在の行がワープじゃないときだけカウント処理
                        if not isWarp then
                            -- カウント
                            local addCount = 0
                            for _,_ in string.gmatch(v, '[CL124]') do
                                addCount = addCount+1
                            end
                            stepsPerSecList[currentVertex] = stepsPerSecList[currentVertex] + addCount
                        end
                    end
                    currentBeat = currentBeat+oneBeat
                end
                lineData = {}
                -- ノートの終了
                if string.find(line, '[;].*') then
                    if secUnit > 1 then
                        for i=1,#stepsPerSecList do
                            -- secUnit=1を基準にスケール調整する
                            stepsPerSecList[i] = 1.0 * stepsPerSecList[i] / secUnit
                        end
                    end
                    break
                end
            end
            if not (line == ',') and not (line == ';') and not (line == ':') then
                lineData[#lineData+1] = split('/', line)[1]
            end
        else
            if extension == 'ssc' then
                -- EDITの時はエディット名も見る
                if string.find(line, '#DESCRIPTION:', 1, true) then
                    discription = string.gsub(line, "^.+:(.*);$", "%1")
                end
                -- スタイルを発見
                if string.find(line, '#STEPSTYPE:'..stepType..';', 1, true) then
                    hasStepType = true
                end
                -- 難易度を発見
                if string.find(line, '#DIFFICULTY:'..difficulty..';', 1, true) then
                    hasDifficulty = true
                end
                -- ノートデータを発見
                if string.find(line, '#NOTES:', 1, true) then
                    -- スタイル、難易度両方が見つかっている場合は取得対象
                    if hasStepType and hasDifficulty then
                        -- ただしEDITの場合はCHARTNAMEもチェック
                        if steps:IsAnEdit() or steps:IsAPlayerEdit() then
                            if (string.upper(steps:GetDescription()) == discription) then
                                readNoteMode = RNM.PROPERTIES
                            end
                        else
                            readNoteMode = RNM.PROPERTIES
                        end
                    end
                    hasStepType   = false
                    hasDifficulty = false
                end
            elseif extension == 'sm' then
                -- スタイルを発見
                if string.find(line, '%s*'..string.gsub(stepType, '-', '%%-')..':.*') then
                    hasStepType = true
                end
                -- 難易度を発見
                if string.find(line, '%s*'..difficulty..':.*') then
                    hasDifficulty = true
                    -- EDITの時はエディット名も見る
                elseif hasStepType and not hasDifficulty then
                    discription = string.gsub(line, "^%s*(.*):.*", "%1")
                end
                -- ノートデータを発見
                if string.find(line, '#NOTES:', 1, true) then
                    hasStepType   = false
                    hasDifficulty = false
                end
                -- スタイル、難易度両方が見つかっている場合は取得対象
                if hasStepType and hasDifficulty then
                    -- ただしEDITの場合はCHARTNAMEもチェック
                    if steps:IsAnEdit() or steps:IsAPlayerEdit() then
                        if (string.upper(steps:GetDescription()) == discription) then
                            readNoteMode = RNM.PROPERTIES
                        end
                    else
                        readNoteMode = RNM.PROPERTIES
                    end
                end
            end
        end
        -- 終端に到達した場合チェック終了
        if file:AtEOF() then
            break
        end
    end
    file:Close()
    file:destroy()
    return stepsPerSecList
end

-- 背景
local function BackgroundActor(height)
    local function BorderActor(c, max)
        local col = color('1, 1, 1, 0.25')
        return LoadActor(W:Object('border'), {
            From = {
                X     = 0,
                Y     = graphH - graphH*c/max,
                Color = col,
            },
            To = {
                X     = graphW,
                Y     = graphH - graphH*c/max,
                Color = col,
            },
        })
    end
    return Def.ActorFrame({
        -- 背景
        Def.Quad({
            InitCommand = function(self)
                self:horizalign(left)
                self:vertalign(bottom)
                self:diffuse(bgColor)
                self:zoomto(graphW, 12 * height)
                self:y(16 * height)
            end
        }),
        Def.Quad({
            InitCommand = function(self)
                self:horizalign(left)
                self:vertalign(bottom)
                self:diffuse(bgColor)
                self:zoomto(graphW, 4 * height)
                self:y(4 * height)
                self:fadetop(bgFade and 1 or 0)
            end
        }),
        BorderActor( 4, 16),
        BorderActor( 8, 16),
        BorderActor(12, 16),
    })
end

-- ノート情報を取得
-- ステマニのエディットを使用せずに特殊な書き方してる場合は基本無視で良さそう？
if not pn or not GAMESTATE:IsPlayerEnabled(pn) then return Def.ActorFrame({}) end
local function GetVertecies(params)
    local alpha = 0.65 + 0.3 * normalizeAlpha
    local function AddVertex(x, ...)
        local value = ...
        local colGB = (not value) and 0 or math.min(math.max((value -12), 0) * 0.0833, 1.0)
        local col = (not value) and color('1, 0, 0, '..alpha) or color(string.format('%.2f, %.2f, %.2f, %.2f', (1-colGB), colGB, colGB, alpha))
        return {
            {x, graphH-math.min((value or 0)*height, max), 0},
            {col[1], col[2], col[3], col[4]*(not value and 0.5 or 1)}
        }
    end
    local stepsList = UpdateGraph(params)
    local vertices = {}
    if not stepsList or #stepsList <= 2 then
        vertices[#vertices+1] = AddVertex(0)
        vertices[#vertices+1] = AddVertex(0, stepsList and stepsList[1] or 0)
        vertices[#vertices+1] = AddVertex(graphW)
        vertices[#vertices+1] = AddVertex(graphW, stepsList and stepsList[1] or 0)
        return vertices
    end
    local songLenSec = (params.LastSec - 0)
    local secUnit = math.ceil(songLenSec / memUnit)  -- 1メモリの秒数
    -- 最後の1メモリは端数になるのでそれを考慮して1メモリの幅を計算
    local oneWidth = (graphW * (1.0*(#stepsList-1)*secUnit/songLenSec))/(#stepsList-1)
    local halfWidth = oneWidth/2
    local curX = 0
    -- 開始位置
    -- 本来ならコメントアウトしている方が正しいがグラフの方が遅れて見えるので半メモリ分手前にずらす
    -- vertices[#vertices+1] = AddVertex(0)
    -- vertices[#vertices+1] = AddVertex(0, stepsList[1]/2)
    -- curX = halfWidth
    -- _LOG(string.format('1メモリ:%3.2f 端数除くサイズ:%3.2f', oneWidth, oneWidth*(#stepsList-1)), true)
    for i=1, #stepsList-1 do
        vertices[#vertices+1] = AddVertex(curX)
        vertices[#vertices+1] = AddVertex(curX, stepsList[i])
        -- _LOG(string.format('現在位置:%3.2f カウント:%d/%d', curX, i, #stepsList-1))
        -- 次の点が最後の点
        if i+1 >= #stepsList then
            -- 次の点の位置がグラフ端を超える＝端数がsecUnit/2以下
            --if curX+oneWidth >= graphW then
            if curX+oneWidth+halfWidth >= graphW then
                vertices[#vertices+1] = AddVertex(graphW)
                vertices[#vertices+1] = AddVertex(graphW, stepsList[i+1])
                -- _LOG(string.format('現在位置:%3.2f <* 端数終端 %3.2f', graphW, curX))
            else
                vertices[#vertices+1] = AddVertex(curX + oneWidth)
                vertices[#vertices+1] = AddVertex(curX + oneWidth, stepsList[i+1])
                -- _LOG(string.format('現在位置:%3.2f カウント:%d/%d', curX + oneWidth, i, #stepsList))
                vertices[#vertices+1] = AddVertex(graphW)
                vertices[#vertices+1] = AddVertex(graphW, stepsList[i+1]/2)
                -- _LOG(string.format('現在位置:%3.2f *> 端数終端 %3.2f', graphW, curX))
            end
            break
        end
        curX = curX + oneWidth
    end
    return vertices
end
return Def.ActorFrame({
    ChangedGameplaySongMessageCommand = function(self, params)
        self:playcommand('UpdateStepSGraph', {
            Vertices = GetVertecies(params),
            Player   = pn,
        })
    end,
    Def.ActorFrame({
        InitCommand = function(self)
            self:xy(-graphW/2, -graphH)
        end,
        BackgroundActor(height),
        -- グラフ（秒）
        Def.ActorMultiVertex({
            InitCommand = function(self)
                self:horizalign(left)
                self:vertalign(top)
                self:SetDrawState({Mode = 'DrawMode_QuadStrip'})
            end,
            UpdateStepSGraphMessageCommand = function(self, params)
                if params.Player ~= pn then return end
                self:SetVertices(1, params.Vertices)
                self:SetNumVertices(#params.Vertices)
            end,
        }),
        Def.ActorMultiVertex({
            InitCommand = function(self)
                self:horizalign(left)
                self:vertalign(top)
                self:SetDrawState({Mode = 'DrawMode_LineStrip'})
                self:SetLineWidth(0.5)
            end,
            UpdateStepSGraphMessageCommand = function(self, params)
                if params.Player ~= pn then return end
                local vertecies = {}
                local i = 1
                for k,v in pairs(params.Vertices) do
                    if (k%2) == 0 then
                        vertecies[i] = {v[1], color('#000000c0')}
                        i = i+1
                    end
                end
                self:SetVertices(1, vertecies)
                self:SetNumVertices(#vertecies)
            end,
        }),
    }),
})