local actorParams = ...
local id     = actorParams['Id'] or nil
-- 全体の大きさ
local width  = actorParams['Width'] or 10
local height = actorParams['Height'] or 10
-- 角の大きさ
local radius = actorParams['Radius'] or 10
-- 色
local color  = actorParams['Color'] or {1.0, 1.0, 1.0, 1.0}
-- 塗りつぶしの有無
local isFill = (actorParams['Fill'] == nil) and true or actorParams['Fill']
-- 描画するフレーム情報（0のフレームは表示しない）
-- 例えば{1, 1, 1, 0, 0, 0, 0, 0, 0}とすれば上辺のみ描画する
local cell   = actorParams['Cell'] or {1, 1, 1, 1, 1, 1, 1, 1, 1}
-- 強制的に縮小画像を使用する（サイズが小さすぎて境界線が消えて見えないときに使用）
local mini   = actorParams['Mini'] or false
-- ブレンドモード
local blend  = actorParams['Blend'] or  'BlendMode_Normal'

-- 座標と基準点の定義
local align = {
    -- 左上
    {h = right,  v = bottom, x = -1, y = -1},
    -- 中上
    {h = center, v = bottom, x =  0, y = -1},
    -- 右上
    {h = left,   v = bottom, x =  1, y = -1},
    -- 左中
    {h = right,  v = middle, x = -1, y =  0},
    -- 中中
    {h = center, v = middle, x =  0, y =  0},
    -- 右中
    {h = left,   v = middle, x =  1, y =  0},
    -- 左下
    {h = right,  v = top,    x = -1, y =  1},
    -- 中下
    {h = center, v = top,    x =  0, y =  1},
    -- 右下
    {h = left,   v = top,    x =  1, y =  1},
}

-- サイズ調整
local function AdjustParams(radius, width, height)
    local r = radius
    -- 角が大きすぎる場合自動修正
    if r > math.min(width/2, height/2) then
        r = math.min(width/2, height/2)
    end
    -- 角を除いたサイズ
    local w = width - r*2
    local h = height - r*2
    return r, w, h
end

-- ID設定
local function SetId(self, newId)
    self.ObjId = newId
end

local r, w, h = AdjustParams(radius, width, height)

local actors = Def.ActorFrame({
    InitCommand = function(self)
        SetId(self, id)
    end,
    UpdateRadiusCommand = function(self, params)
        -- IDが意図的に指定されている場合のみ更新キャンセルチェック（通常は不要）
        if params.Id and params.Id ~= self.ObjId then return end
        -- 指定がある場合のみ更新
        params.Width  = params.Width or width
        params.Height = params.Height or height
        params.Radius = params.Radius or radius
        params.Color  = params.Color or color
        width = params.Width
        height = params.Height
        radius = params.Radius
        color = params.Color
        self:playcommand('UpdateRadiusChild', params)
    end,
})

for i=1, #align do
    if cell[i] > 0 then
        actors[#actors+1] = LoadActor(isFill and 'radius' or ((w >= 24 and h >= 24 and not mini) and 'border') or 'mini-border')..{
            InitCommand = function(self)
                self:animate(false)
                self:setstate(i-1)
                self:horizalign(align[i].h)
                self:vertalign(align[i].v)
                self:xy(w/2*align[i].x, h/2*align[i].y)
                self:zoomto((align[i].x == 0) and w or r, (align[i].y == 0) and h or r)
                self:diffuse(color)
                self:blend(blend)
            end,
            UpdateRadiusChildCommand = function(self, params)
                self:stoptweening()
                r, w, h = AdjustParams(params.Radius, params.Width, params.Height)
                -- Waitが0より大きい場合アニメーション
                if params.Wait and params.Wait > 0 then
                    self:linear(params.Wait or 0)
                end
                self:xy(w/2*align[i].x, h/2*align[i].y)
                self:zoomto((align[i].x == 0) and w or r, (align[i].y == 0) and h or r)
                self:diffuse(params.Color)
            end,
        }
    end
end

return {
    Id    = SetId,
    Actor = function() return actors end,
}