-- サイドライトの中身
local width, baseHeight, targetPn, opt = ...

local min    = baseHeight * ((opt and opt.Min) or 0.1)
local height = baseHeight - min
local alpha  = (opt and opt.Alpha) or 0.8
local pn     = GAMESTATE:IsPlayerEnabled(targetPn) and targetPn or GAMESTATE:GetMasterPlayerNumber()
local horiz  = (opt and opt.Horiz) and opt.Horiz or (W:IsP1(targetPn) and left or right)

local reso  = 100
local speed = 50

local height_01 = height * 0.1
local height_09 = height - height_01

local danger = tonumber(THEME:GetMetric('LifeMeterBar', 'DangerThreshold'))

local totalDelta = 0
local screenName = ''
local function update(self, delta)
    totalDelta = totalDelta + delta
    self:playcommand('UpdateSideLight', {
        Delta  = totalDelta,
        Screen = screenName,
        Second = GAMESTATE:GetSongPosition():GetMusicSeconds(),
        Song   = (screenName == 'Game') and GAMESTATE:GetCurrentSong() or nil,
    })
end

local function SideLightActor(name, initColor)
    return Def.Quad({
        Name = name,
        InitCommand = function(self)
            self:horizalign(horiz)
            self:vertalign(bottom)
            self:zoomto(width, baseHeight)
            self:diffuse(initColor)
        end,
    })
end

local t = Def.ActorFrame({
    -- 外でInitCommandを使われたら上書きされてしまうのでDef.ActorFrameで囲む
    Def.ActorFrame({
        InitCommand = function(self)
            self:SetUpdateFunction(update)
            self:xy(0, baseHeight/2)
        end,
        -- 現在のスクリーンを取得する
        OnCommand = function(self)
            local screen = SCREENMAN:GetTopScreen()
            if screen.Center1Player then
                screenName = 'Game'
            else
                screenName = 'Menu'
            end
        end,
        -- ベース
        Def.ActorFrame({
            InitCommand = function(self)
                self:zoomx(1)
            end,
            SideLightActor('Base', color('0, 0, 0, 1')),
        }),
        -- ライト
        SideLightActor('Overlay', color('1, 1, 1, 0'))..{
            InitCommand = function(self)
                self:blend('BlendMode_Add')
            end,
            UpdateSideLightCommand = function(self, params)
                local size = 0
                local col  = nil
                -- ゲーム中
                if params.Song then
                    -- クリア表記時は点滅
                    if params.Second > params.Song:GetLastSecond() then
                        size = height
                        col  = HSVA(math.floor(params.Delta*speed) % 360, 1, 1, 
                                ((math.floor(params.Delta * 10) % 5) < 2) and 0 or alpha)
                    -- ゲーム中通常表記
                    else
                        local timing = GAMESTATE:GetCurrentSteps(pn):GetTimingData()
                        local beat   = (timing:GetBeatFromElapsedTime(params.Second) * 100) % 400
                        local life   = SCREENMAN:GetTopScreen():GetLifeMeter(pn)
                        local curLife = life:GetLife()
                        -- ライフゲージでサイズ調整
                        local drawHeight = 0
                        if curLife > danger then
                            drawHeight = height_01 + height_09 * (curLife-danger) / (1-danger)
                        else
                            drawHeight = height_01 + height_09 * (danger - curLife) / danger
                        end
                        -- ビートでサイズ調整
                        if beat < 200 then
                            size = drawHeight * beat / 200
                        else
                            size = drawHeight * (400 - beat) / 200
                        end
                        -- カラーリング
                        if curLife >= 1 then
                            col = HSVA(math.floor(params.Delta*speed) % 360, 1, 1, alpha)
                        elseif life:IsInDanger() or life:IsFailing() or (life.GetLivesLeft and life:GetLivesLeft() <= 1) then
                            col = color('1.0, 0, 0, '..alpha)
                        else
                            col = color('0, 1.0, 1.0, '..alpha)
                        end
                    end
                -- その他
                else
                    size = height * (math.floor(params.Delta*speed) % reso) / reso
                    col  = HSVA(math.floor(params.Delta*speed) % 360, 1, 1, alpha)
                end
                self:zoomto(width, min + size * (reso + 1) / reso)
                self:diffuse(col)
            end,
        },
    }),
})

return t