local params = ...
local isSmallText = params.IsSmallText == nil and false or params.IsSmallText
local maxwidth    = (params and params.Width) and params.Width or SCREEN_WIDTH
local textColor   = (params and params.Color) and params.Color or color('#ffffffff')
local strokeColor = (params and params.Stroke) and params.Stroke or nil
local mainZoom    = (params and params.MainZoom) and params.MainZoom or (1.2 - 0.3 * (853.0 - math.max(math.min(SCREEN_WIDTH, 853), 640)) / 253)
local isCenter    = not (not params or not params.IsCenter)
local text        = ''
local order       = ''

-- テキスト
local function TextActor(actorName, font, zoom)
    return LoadFont(font)..{
        Name = actorName,
        OnCommand = function(self)
            self:horizalign(isCenter and center or left)
            --self:zoomy(zoom)
            if strokeColor then
                self:strokecolor(strokeColor)
            end
        end,
        -- 曲情報の変更
        ChangedSongMessageCommand = function(self, params)
            local name = ''
            self:y(0)
            if actorName == 'MainTitle' then
                if params.Song then
                    name = params.Song:GetDisplayMainTitle()
                    self:vertalign(bottom)
                    self:y((params.Song:GetDisplaySubTitle() == '') and -1 or -8)
                elseif params.Name then
                    self:vertalign(middle)
                    -- カスタムソート時のみgroup.iniのNAMEをチェック
                    if order ~= '' and ToEnumShortString(order) == 'Preferred' then
                        name = YA_GROUP:GroupName(params.Name)
                    else
                        name = params.Name
                    end
                end
            elseif actorName == 'SubTitle' then
                self:y(5)
                if params.Song then
                    name = params.Song:GetDisplaySubTitle()
                end
            else
                if params.Song then
                    name = params.Song:GetDisplayArtist()
                    self:vertalign(top)
                    self:y((params.Song:GetDisplaySubTitle() == '') and 12 or 16)
                end
            end
            self:settext(name or '')
            self:playcommand('UpdateText', {
                ActorName = self:GetName(),
            })
        end,
        -- テキストスタイルの変更
        UpdateTextMessageCommand = function(self, params)
            if params.ActorName and params.ActorName ~= actorName then
                return
            end
            self:visible(false)
            local curZoomX = self:GetZoomX()
            self:zoomx(1)
            mainZoom = params.MainZoom or mainZoom
            local nextZoom = (actorName == 'MainTitle') and mainZoom or zoom
            local textWidth = self:GetWidth() * nextZoom
            local nextWidth = (textWidth > maxwidth) and maxwidth / textWidth * nextZoom or nextZoom
            self:zoomx(curZoomX)
            self:zoomy(zoom)
            self:visible(true)
            if params.Sleep then
                self:sleep(params.Sleep)
            end
            if params.Anime then
                self:linear(params.Anime)
            end
            self:zoomx(nextWidth)
            self:zoomy(nextZoom)
            self:diffuse(textColor)
            if strokeColor then
                self:strokecolor(strokeColor)
            end
        end,
    }
end

return Def.ActorFrame({
    Def.Actor({
        OnCommand = function(self)
            order = GAMESTATE:GetSortOrder() or ''
        end,
        UpdateTextMessageCommand = function(self, params)
            -- 再設定されていたら情報を更新する
            text        = (params and params.Text) and params.Text or text
            maxwidth    = (params and params.Width) and params.Width or maxwidth
            textColor   = (params and params.Color) and params.Color or textColor
            strokeColor = (params and params.Stroke) and params.Stroke or strokeColor
            mainZoom    = (params and params.MainZoom) and params.MainZoom or mainZoom
        end,
    }),
    -- 16:9以上の横幅は1.2、4:3以下の横幅は0.9として、その間の横幅の場合は調整する
    TextActor('MainTitle', isSmallText and 'Common Small Normal' or 'Common Normal', mainZoom),
    TextActor('SubTitle',  'Common Small Normal', 0.5),
    TextActor('Artist',    isSmallText and 'Common Small Normal' or 'Common Normal', 0.65),
})