-- 開始前に表示するターゲットスコア
local mode = ...
local areaSpace = math.min(200, (SCREEN_WIDTH - 320)/2)
local areaWidth = 200
local areaHeight = 120
local function TargetActor(pn, index, name, mini)
    return LoadActor(W:Object('target/static/static'), pn, name, mini, false)..{
        InitCommand = function(self)
            if index == 1 then
                self:xy(0, -areaHeight/8+5)
                self:zoom(1)
            else
                self:xy((index%2) * areaWidth/2 - areaWidth/4, areaHeight/4+5)
                self:zoom(0.8)
            end
        end,
    }
end
local t = Def.ActorFrame({
    OnCommand = function(self)
        if mode == 'Gameplay' then
            self:playcommand('LoadTargetScore')
            self:sleep(0.4)
            self:sleep(W_TIMER:GameJacketSleep()-0.2)
            self:queuecommand('HideTargetScore')
        else
            self:playcommand('LoadTargetScore')
        end
        --self:queuecommand('ShowTargetScore')
    end,
    OffCommand = function(self)
        if mode == 'SelectMusic' then
            self:playcommand('LoadTargetScore')
            self:queuecommand('ShowTargetScore')
        end
    end,
})

local function PlayerActor(pn)
    local boxHeight = areaHeight

    local target = W_PLAYER:ReadPref(pn, 'TargetList')
    local pnSetting = W_PLAYER:ReadPref(pn, 'EnableTarget')
    if pnSetting ~= 'On' then return Def.ActorFrame({}) end
    if not target then target = '' end
    local targetList = split('|', target or '')
    -- 設定ファイルは存在するが、全ターゲット設定なしの場合はハイスコアのみ表示する
    if string.gsub(target, '|', '') == '' and pnSetting == 'On' then
        targetList[1] = W_TARGET:List()[1]
    end

    -- ターゲットスコアの表示が1行で良い場合は縦幅を調整する
    local isOneLine = false
    if (targetList[1] == '') or ((not targetList[2] or targetList[2] == '') and (not targetList[3] or targetList[3] == '')) then
        boxHeight = 80
        isOneLine = true
    end

    local actors = Def.ActorFrame({
        InitCommand = function(self)
            self:x(W:IsP1(pn) and SCREEN_LEFT + areaSpace/2 or SCREEN_RIGHT - areaSpace/2)
            self:y(W_PLAYER:IsReverse(pn) and SCREEN_TOP + areaHeight or SCREEN_HEIGHT - areaHeight)
            self:zoom((areaSpace < 200) and (1.0*areaSpace/200) or 1)
        end,
        -- 選曲画面は初期非表示にする
        OnCommand = function(self)
            if mode == 'SelectMusic' then
                self:addx(W:IsP1(pn) and -areaSpace or areaSpace)
                self:diffusealpha(0)
            end
        end,
        -- 表示
        ShowTargetScoreCommand = function(self)
            self:y(W_PLAYER:IsReverse(pn) and SCREEN_TOP + areaHeight or SCREEN_HEIGHT - areaHeight)
            self:diffusealpha(1)
            self:sleep(0.4)
            self:decelerate(0.2)
            self:addx(W:IsP1(pn) and areaSpace or -areaSpace)
        end,
        -- 非表示
        HideTargetScoreMessageCommand = function(self)
            self:decelerate(0.2)
            self:addx(W:IsP1(pn) and -areaSpace or areaSpace)
            -- 残ってると処理が重いので非表示にする
            self:sleep(0.01)
            self:diffusealpha(0)
        end,
        -- 読みこんで反映
        LoadTargetScoreCommand = function(self)
            local playerScore, tmp, playerIsNoPlay = W_SCORE:HighDPScore(pn, false)
            self:playcommand('SetPlayerScore', {
                Score    = playerScore,
                IsNoPlay = playerIsNoPlay,
            })
        end,
        -- WAIEI Coreのスクロール方向変更メッセージ
        ChangeReverseMessageCommand = function(self, params)
            if params.Player ~= pn then return end
            self:stoptweening()
            self:linear(0.2)
            self:y((params.New >= 0.5) and SCREEN_TOP + boxHeight or SCREEN_HEIGHT - boxHeight)
        end,
    })

    actors[#actors+1] = Def.ActorFrame({
        Def.Quad({
            InitCommand = function(self)
                self:zoomto(areaWidth, 20)
                self:diffuse(BoostColor(W_PLAYER:Color(pn), 0.75))
                self:y(-boxHeight/2+10)
                if W:IsP1(pn) then
                    self:fadeleft(0.8)
                else
                    self:faderight(0.8)
                end
            end,
        }),
        LoadActor(W:Object('boxshadow'), 'TargetArea'..pname(pn), {
            Center = color('0.2, 0.2, 0.2, 0.8'),
            Length = 10,
            Color  = color('0.1, 0.1, 0.1, 0.5'),
        })..{
            InitCommand = function(self)
                self:playcommand('UpdateBoxShadow', {
                    Id     = 'TargetArea'..pname(pn),
                    Width  = areaWidth,
                    Height = boxHeight,
                })
            end,
        },
        W_AVATAR:Frame(W_PLAYER:PlayerAvatar(pn), 'TaregetScorePlayer'..pname(pn), {
            Flip = W:IsP1(pn),
            Mini = true,
            Color = BoostColor(W_PLAYER:Color(pn), 0.66),
            Type  = 0,
            RandomNum = W_AVATAR:RandomNum(W_PLAYER:Name(pn)),
        })..{
            InitCommand = function(self)
                self:xy((areaWidth/2-25) * (W:IsP1(pn) and -1 or 1), -boxHeight/2)
                self:zoom(0.4)
            end,
        },
        -- 自身のハイスコア
        Def.ActorFrame({
            InitCommand = function(self)
                self:x(25 * (W:IsP1(pn) and 1 or -1))
                self:y(-boxHeight/2 + 8)
            end,
            LoadFont('Common Small Bold')..{
                Text = 'HIGHSCORE',
                InitCommand = function(self)
                    self:horizalign(left)
                    self:vertalign(top)
                    self:zoom(0.5)
                    self:xy(-areaWidth/2 + 30, 0)
                end,
            },
            LoadFont('Common Normal')..{
                Text = '',
                InitCommand = function(self)
                    self:horizalign(right)
                    self:vertalign(top)
                    self:zoom(0.6)
                    self:xy(areaWidth/2 - 30, -1)
                end,
                SetPlayerScoreCommand = function(self, params)
                    if params.IsNoPlay then
                        self:settext('NO PLAY')
                    else
                        self:settext(W_STRING:DP(0.0001 * math.floor(params.Score*100), false))
                    end
                end,
            },
        }),
    })
    for i=1, #targetList do
        if i > 3 then break end
        actors[#actors+1] = TargetActor(pn, i, targetList[i], i ~= 1) ..{
            InitCommand = function(self)
                -- メインターゲットが設定されていない場合はメインターゲットの位置にサブターゲットを表示
                if isOneLine then
                    self:y(10)
                end
            end
        }
    end
    return actors
end

for pn in ivalues(PlayerNumber) do
    if W_PLAYER:IsEnabled(pn) then
        t[#t+1] = PlayerActor(pn)
    end
end
return t
