-- 単純にハイスコアとのスコア差を表示するだけバージョン
local pn, targetFileName, mini, isEvaluation = ...
if targetFileName == '' then
    return Def.ActorFrame({})
end

local isNoPlay = false

local targetMaxScore = 0
local targetMaxDP = 0
local isSystem, systemName = W_TARGET:IsSystem(targetFileName)
local targetName = W_TARGET:Name(targetFileName)

local function TargetGroupActor(pn)
    local avatarPath = W_TARGET:Avatar(targetFileName)
    if isSystem and systemName == 'HighScore' then
        avatarPath = W_PLAYER:PlayerAvatar(pn)
    end
    return Def.ActorFrame({
        Name = 'Group',
        W_AVATAR:Actor(avatarPath, 'Target', {
            Mini  = mini,
            Flip  = false,
            Type  = 0,
            -- ハイスコアが対象の場合、自分自身の名前でランダムカラーを設定する
            RandomNum = W_AVATAR:RandomNum((not isSystem or systemName ~= 'HighScore') and targetName or W_PLAYER:Name(pn)),
        })..{
            Name = 'TargetAvatar',
            OnCommand = function(self)
                self:zoom(0.3)
                self:xy(mini and 35 or 75, 0)
            end,
        },
        LoadFont('Common Normal')..{
            InitCommand = function(self)
                self:horizalign(right)
                self:xy(mini and 10 or 50, mini and -8 or 8)
                self:diffuse(1, 1, 1, 1)
                self:strokecolor(Color('Outline'))
                self:zoom(0.5)
                self:settext('')
            end,
            -- 自身のスコアを設定
            SetTargetScoreCommand = function(self, params)
                if isNoPlay then
                    self:settext('NO PLAY')
                else
                    self:settext(W_STRING:DP(0.0001 * math.floor(params.Target*100), false))
                end
                self:diffuse(params.Color)
            end,
        },
        LoadFont('Common Normal')..{
            InitCommand = function(self)
                self:horizalign(right)
                self:xy(mini and 10 or -10, 8)
                self:diffuse(0.9, 0.9, 0.9, 1)
                self:strokecolor(Color('Outline'))
                self:zoom(0.5)
                self:maxwidth((mini and 65 or 85)/0.5)
                self:settext(targetName)
            end,
        },
        LoadFont('Common Small Bold')..{
            Text = 'TARGET',
            InitCommand = function(self)
                self:horizalign(left)
                self:xy(-90, -12)
                self:diffuse(1, 1, 1, 0.85)
                --self:strokecolor(Color('Outline'))
                self:zoom(0.5)
                self:visible(not mini)
            end,
        },
    })
end

local function TargetActor(pn)
    local boxW, boxH = 120, 40
    return Def.ActorFrame({
        Name = 'Target',
        Def.Quad({
            InitCommand = function(self)
                self:y(0)
                self:zoomto(boxW-10, boxH+5)
                self:fadeleft(0.5)
                self:faderight(0.5)
            end,
            -- 自身のスコアを設定
            SetTargetScoreCommand = function(self, params)
                local col = color('0, 0, 0, 0.5')
                if not isNoPlay then
                    local lead = W_TARGET:DiffToLead(params.Diff)
                    if lead ~= 0 then
                        col = BoostColor(params.Color, 0.5)
                        col[4] = 0.66
                    end
                end
                self:diffuse(col)
            end,
        }),
        TargetGroupActor(pn),
    })
end

return Def.ActorFrame({
    SetPlayerScoreCommand = function(self, params)
        local playerScore = params.Score
        if isSystem then
            if systemName == 'HighScore' then
                -- HighScoreがターゲット
                local tmp
                targetMaxScore, tmp, isNoPlay = W_SCORE:HighDPScore(pn, isEvaluation)
            elseif string.match(systemName, '^%d+%%$') then
                -- DancePoints(%)がターゲット
                targetMaxScore = tonumber(string.match(systemName, '^(%d+)%%$'))
                isNoPlay = false
            else
                targetMaxScore = 0
                isNoPlay = true
            end
        else
            targetMaxScore, targetMaxDP, isNoPlay = W_TARGET:GetScore(targetFileName, pn)
        end
        local diff, col = W_STRING:DiffAndColor(playerScore, targetMaxScore, isNoPlay)
        self:playcommand('SetTargetScore', {
            Self   = playerScore,
            Target = targetMaxScore,
            Player = pn,
            Diff   = diff,
            Color  = col,
        })
        -- 表情変更
        local targetAvatar = self:GetChild('Target'):GetChild('Group'):GetChild('TargetAvatar')
        targetAvatar:playcommand('DefaultEmotion',{})
    end,
    TargetActor(pn),
})