--[[
    スクロールバー付きリスト
    初期化時にDataに文字列のテーブルを渡すことでその内容のリストを作成
    BroadcastのUpdateScrollDataMessageで更新が可能
    ただし、最初に渡したデータ以上の項目数にはできない
    もし、初期値以上のデータを渡したい場合は、初期化時にBufferで数値を設定しておくことで
    その数値分の項目数まであとから設定することが可能
    リストのスクロールはBroadcastのScrollMenuMessageでCursorを設定することで可能
--]]
local params = ...
local id = params.Id or nil
local width  = params.Width or 220
local height = params.Height or 360
local lineHeight = params.Line or 30
local listData   = params.Data or {}
local zoom = 0.9*lineHeight/30
local buffer = params.Buffer or #listData

local function ListActor()
    local actors = Def.ActorFrame({})
    for i=1, buffer do
        actors[#actors+1] = Def.ActorFrame({
            InitCommand = function(self)
                self:zoom((i <= #listData) and zoom or 0)
                self:y((i-1)*lineHeight)
            end,
            UpdateScrollDataMessageCommand = function(self, params)
                if params.Id and params.Id ~= id then
                    return
                end
                listData = params.Data
                self:zoom((i <= #listData) and zoom or 0)
                self:playcommand('UpdateText', params)
            end,
            LoadFont('Common Normal')..{
                Name = 'ScrollText'..i,
                Text = listData[i] or '',
                InitCommand = function(self)
                    self:maxwidth((width - 20)/zoom)
                    self:diffuse(Color('White'))
                    self:strokecolor(Color('Outline'))
                end,
                UpdateListMessageCommand = function(self, params)
                    if params.Active ~= nil and params.Active == false then
                        self:diffusealpha(0.5)
                    else
                        self:diffusealpha(1)
                    end
                end,
                UpdateTextCommand = function(self, params)
                    self:settext(params.Data[i] or '')
                end,
            },
        })
    end
    return actors
end

return Def.ActorFrame({
    Def.ActorFrame({
        LoadActor(W:Object('boxshadow'), 'bg', {
            Length = 30,
            Color = color('0, 0, 0, 0.25'),
            Center = color('0.01, 0.08, 0.1, 0.85'),
        })..{
            OnCommand = function(self)
                self:playcommand('UpdateBoxShadow', {
                    Id = 'bg',
                    Width = width,
                    Height = height,
                })
            end,
        },
        Def.Quad({
            Name = 'SelectBG',
            InitCommand = function(self)
                self:diffuse(1, 1, 1, 0.5)
                self:zoomto(width-20, lineHeight)
                self:fadeleft(0.5)
                self:faderight(0.5)
            end,
            UpdateListMessageCommand = function(self, params)
                if params.Active ~= nil and params.Active == false then
                    self:fadeleft(0)
                    self:faderight(0)
                    self:diffusealpha(0.25)
                else
                    self:fadeleft(0.5)
                    self:faderight(0.5)
                    self:diffusealpha(0.5)
                end
            end,
        }),
        Def.Quad({
            OnCommand=function(self)
                self:zoomto(width, SCREEN_HEIGHT)
                self:vertalign(bottom)
                self:y(-height/2)
                self:clearzbuffer(true)
                self:zwrite(true)
                self:blend('BlendMode_NoEffect')
            end,
        }),
        Def.Quad({
            OnCommand=function(self)
                self:zoomto(width, SCREEN_HEIGHT)
                self:vertalign(top)
                self:y(height/2)
                self:zwrite(true)
                self:blend('BlendMode_NoEffect')
            end,
        }),
        ListActor()..{
            OnCommand=function(self)
                self:ztest(true)
            end,
            ScrollMenuMessageCommand = function(self, params)
                if params.Id and params.Id ~= id then
                    return
                end
                self:stoptweening()
                if not params.Init then
                    self:linear(0.1)
                end
                self:y(-(params.Cursor-1) * lineHeight)
            end,
        },
        -- スクロールバー
        LoadActor(W:Object('scrollbar'), {
                Width  = 6,
                Height = height - 5,
                Count  = #listData,
            })..{
            OnCommand = function(self)
                self:xy(width/2, 0)
            end,
        },
        -- マスクのクリア
        Def.Quad({
            OnCommand = function(self)
                self:clearzbuffer(true)
                self:zoom(0)
            end,
        }),
    }),
})