local customList = W_CUSTOM:CustomList()
local listData = {THEME:GetString('ScreenCustom', 'NotFoundList')}
for i=1, #customList do
    listData[i] = customList[i] and customList[i].Title or THEME:GetString('ScreenCustom', 'NotFoundList')
end

local function LabelActor(text)
    return LoadFont('Common Small Normal')..{
        Text = text,
        InitCommand = function(self)
            self:horizalign(left)
            self:vertalign(top)
            self:zoom(0.4)
            self:wrapwidthpixels(360/0.4)
            self:maxwidth(360/0.4)
            self:diffuse(Color('White'))
            self:strokecolor(Color('Outline'))
        end,
    }
end

local function TextActor(text)
    return LoadFont('Common Small Normal')..{
        Text = text,
        InitCommand = function(self)
            self:horizalign(left)
            self:vertalign(top)
            self:vertspacing(10)
            self:zoom(0.55)
            self:wrapwidthpixels(360/0.55)
            self:maxwidth(360/0.55)
            self:diffuse(Color('White'))
            self:strokecolor(Color('Outline'))
        end,
        ScrollMenuMessageCommand = function(self, params)
            self:stoptweening()
            self:diffusealpha(0)
            self:linear(0.1)
            self:diffusealpha(1)
        end,
    }
end
return Def.ActorFrame({
    InitCommand = function(self)
        W_CUSTOM:LoadLang()
    end,
    Def.Actor({
        OnCommand = function(self)
            MESSAGEMAN:Broadcast('GetCustomList', {List = customList})
        end,
    }),
    Def.ActorFrame({
        W_CUSTOM:Template('list', {
            Width  = 220,
            Height = 300,
            Line   = 30,
            Data   = listData,
        })..{
            InitCommand = function(self)
                self:xy(-200, 30)
            end,
        },

        LoadActor(W:Object('boxshadow'), 'title', {
            Length = 30,
            Color = color('0, 0, 0, 0.25'),
            Center = color('0.01, 0.08, 0.1, 0.85'),
        })..{
            OnCommand = function(self)
                self:xy(115, -95)
                self:playcommand('UpdateBoxShadow', {
                    Id = 'title',
                    Width = 390,
                    Height = 50,
                })
            end,
        },
    
        LoadActor(W:Object('boxshadow'), 'body', {
            Length = 30,
            Color = color('0, 0, 0, 0.25'),
            Center = color('0.01, 0.08, 0.1, 0.85'),
        })..{
            OnCommand = function(self)
                self:xy(115, 60)
                self:playcommand('UpdateBoxShadow', {
                    Id = 'body',
                    Width = 390,
                    Height = 240,
                })
            end,
        },
    }),
    LoadFont('Common Normal')..{
        Text = customList[1] and customList[1].Title or THEME:GetString('ScreenCustom', 'NotFoundList'),
        InitCommand = function(self)
            self:xy(110, -95)
            self:maxwidth(360)
            self:wrapwidthpixels(360)
            self:diffuse(Color('White'))
            self:strokecolor(Color('Outline'))
        end,
        ScrollMenuMessageCommand = function(self, params)
            self:stoptweening()
            self:settext(customList[params.Cursor] and customList[params.Cursor].Title or '')
            self:diffusealpha(0)
            self:linear(0.1)
            self:diffusealpha(1)
        end,
    },
    -- Description
    LabelActor(THEME:GetString('ScreenCustom', 'LabelDescription'))..{
        InitCommand = function(self)
            self:xy(-70, -48)
        end,
    },
    TextActor(customList[1] and customList[1].Description or THEME:GetString('ScreenCustom', 'NotFoundText'))..{
        InitCommand = function(self)
            self:xy(-60, -30)
        end,
        ScrollMenuMessageCommand = function(self, params)
            self:settext(customList[params.Cursor] and customList[params.Cursor].Description or '')
        end,
    },
    -- Version
    LabelActor(THEME:GetString('ScreenCustom', 'LabelVersion'))..{
        InitCommand = function(self)
            self:xy(-70, 102)
        end,
    },
    TextActor(customList[1] and customList[1].Version or '')..{
        InitCommand = function(self)
            self:xy(-60, 120)
        end,
        ScrollMenuMessageCommand = function(self, params)
            self:settext(customList[params.Cursor] and customList[params.Cursor].Version or '')
        end,
    },
    -- バージョン警告
    TextActor('')..{
        InitCommand = function(self)
            self:xy(-60, 145)
            self:diffuse(color('#FF6464FF'))
            self:zoom(0.45)
            self:wrapwidthpixels(360/(0.55*0.45))
            self:maxwidth(360/0.45)
        end,
        ScrollMenuMessageCommand = function(self, params)
            self:visible(W:Ver() < W:Ver(split(' ', customList[params.Cursor].WAIEI)))
            self:settextf(THEME:GetString('ScreenCustom', 'NewVersion'), customList[params.Cursor].WAIEI, W:StrVer())
        end,
    },

    -- 画面上部の説明
    LoadActor('description')..{
        InitCommand = function(self)
            self:y(-165)
        end,
    },
    W_CUSTOM:Template('Helper', {
        Text = THEME:GetString('ScreenCustom', 'HelperText'),
    }),
})
