local mpn = GAMESTATE:GetMasterPlayerNumber()
local size = 200
local courseBnY = size*164/418

local function GraphicActor()
    return Def.Sprite({
        OnCommand = function(self)
            if GAMESTATE:IsCourseMode() then
                -- コースモードはバナー
                local course = GAMESTATE:GetCurrentCourse()
                --self:LoadBackground(W_GRAPHICS:CourseJacket(course))
                self:LoadBackground(W_GRAPHICS:CourseBanner(course))
                self:scaletofit(-size/2+1, -courseBnY/2+1, size/2-1, courseBnY/2-1)
                self:y(-size/2 + courseBnY/2)
            else
                local song = W:Song()
                self:LoadBackground(W_GRAPHICS:SongJacket(song))
                self:scaletofit(-size/2+1, -size/2+1, size/2-1, size/2-1)
            end
        end,
    })
end

local function CourseDetailActor()
    if not GAMESTATE:IsCourseMode() then
        return Def.Actor({})
    end
    local trails = GAMESTATE:GetCurrentTrail(mpn):GetTrailEntries()
    local songList = Def.ActorFrame({})
    for i=1, #trails do
        if i > 1 then
            songList[#songList+1] = Def.Quad({
                OnCommand = function(self)
                    self:zoomto(size, 1)
                    self:y(5 + (i-1)*20)
                    self:diffuse(1, 1, 1, 0.75)
                    self:fadeleft(1)
                    self:faderight(1)
                end,
            })
        end
        if i >= 5 then
            songList[#songList+1] = LoadFont('Common Small Normal')..{
                Text = 'And more',
                OnCommand = function(self)
                    self:horizalign(right)
                    self:zoom(0.3)
                    self:xy(size/2-10, 95)
                end,
            }
            break
        end
        local song = trails[i]:GetSong()
        local steps = trails[i]:GetSteps()
        songList[#songList+1] = LoadFont('Common Small Normal')..{
            Text = W_STRING:CutWidth(song:GetDisplayFullTitle(), 20),
            OnCommand = function(self)
                self:horizalign(left)
                self:zoom(0.35)
                self:maxwidth((size/2+5)/0.35)
                self:xy(-size/2 + 10, 15 + (i-1)*20)
                self:diffuse(YA_GROUP:MenuColor(song))
                self:strokecolor(Color('Black'))
            end,
        }
        songList[#songList+1] = LoadFont('Common Small Bold')..{
            Text = W_DIF:Name(steps:GetDifficulty()),
            OnCommand = function(self)
                self:horizalign(left)
                self:zoom(0.3)
                self:maxwidth(50/0.3)
                self:xy(20, 15 + (i-1)*20-1)
                self:diffuse(W_DIF:Color(steps:GetDifficulty()))
                self:strokecolor(Color('Black'))
            end,
        }
        songList[#songList+1] = LoadFont('Common Small Bold')..{
            Text = steps:GetMeter(),
            OnCommand = function(self)
                self:horizalign(right)
                self:zoom(0.5)
                self:maxwidth(20/0.5)
                self:xy(size/2 - 10, 15 + (i-1)*20)
                self:diffuse(W_DIF:Color(steps:GetDifficulty()))
                self:strokecolor(Color('Black'))
            end,
        }
    end
    return Def.ActorFrame({
        songList,
        LoadFont('Common Small Normal')..{
            Text = #trails..' Song'..((#trails > 1) and 's' or ''),
            OnCommand = function(self)
                self:zoom(0.4)
                self:y(110)
            end,
        },
    })
end

return Def.ActorFrame({
    -- 全体
    LoadActor(W:Object('boxshadow'), 'BannerFrame', {
        Length = 20,
        Color  = {0, 0, 0, 0.4},
        Center = {0, 0, 0, 0.8},
    })..{
        OnCommand = function(self)
            self:y(20)
            self:playcommand('UpdateBoxShadow', {
                Id         = 'BannerFrame',
                Width      = size,
                Height     = size + 40,
            })
        end,
    },
    -- ジャケット
    GraphicActor(),
    -- 曲名部分
    Def.ActorFrame({
        OnCommand = function(self)
            self:y(GAMESTATE:IsCourseMode() and (40+courseBnY)/2 or 120)
            self:playcommand('ChangedMenuColor', {
                Color = GAMESTATE:IsCourseMode()
                            and Color('White')
                            or YA_GROUP:MenuColor(W:Song()),
            })
        end,
        Def.Quad({
            InitCommand = function(self)
                self:zoomto(size-2, GAMESTATE:IsCourseMode() and (size+40)-courseBnY-2 or 40-2)
            end,
            ChangedMenuColorCommand = function(self, params)
                self:diffuse(params.Color)
            end,
        }),
        Def.Quad({
            InitCommand = function(self)
                self:diffuse(0, 0, 0, 0.85)
                self:zoomto(size, GAMESTATE:IsCourseMode() and (size+40)-courseBnY or 40)
            end,
        }),
        Def.Quad({
            InitCommand = function(self)
                self:zoomto(size-2, 1)
                self:y(GAMESTATE:IsCourseMode() and -(((size+40)-courseBnY)/2-1) or -19)
                self:faderight(0.9)
            end,
            ChangedMenuColorCommand = function(self, params)
                self:diffuse(params.Color)
            end,
        }),
        Def.Quad({
            InitCommand = function(self)
                self:zoomto(size-2, 1)
                self:y(GAMESTATE:IsCourseMode() and (((size+40)-courseBnY)/2-1) or 19)
                self:fadeleft(0.9)
            end,
            ChangedMenuColorCommand = function(self, params)
                self:diffuse(params.Color)
            end,
        }),
        -- 曲名
        LoadActor(W:Object('song title'),{
            IsCenter    = true,
            IsSmallText = true,
            MainZoom    = 0.9,
            Stroke      = color('0, 0, 0, 0.5'),
            Width       = (size-10)/0.65,
        })..{
            OnCommand = function(self)
                self:zoom(0.65)
                self:y(GAMESTATE:IsCourseMode() and -((size+40)-courseBnY)/2+20 or -2)
                if GAMESTATE:IsCourseMode() then
                    local course = GAMESTATE:GetCurrentCourse()
                    local name = ''
                    if course then
                        name = course:GetTranslitFullTitle()
                        local match = string.match(name, '(.+) Random$')
                        if match then
                            name = YA_GROUP:GroupName(match)..' Random'
                        end
                    end
                    self:playcommand('ChangedSong', {
                        Name = name,
                    })
                else
                    self:playcommand('ChangedSong', {
                        Song = W:Song(),
                    })
                end
            end,
        },
        CourseDetailActor()..{
            OnCommand = function(self)
                self:y(-((size+40)-courseBnY)/2+40)
            end,
        },
    }),
    -- スタイル
    LoadActor(W:Object('style'))..{
        OnCommand=function(self)
            self:y(-size/2-15)
            self:playcommand('CallbackSteps', {
                Song      = W:Song(),
                Steps     = W:Steps(GAMESTATE:GetMasterPlayerNumber()),
                Player    = GAMESTATE:GetMasterPlayerNumber(),
                StepsType = GAMESTATE:GetCurrentStyle():GetStepsType(),
            })
        end,
    },
})
