local bgColor = ...
local protrusionW = 100
local corner = {W = 30, H = 14}
local region = {
    {-SCREEN_CENTER_X,        0,         0},
    {-protrusionW - corner.W, 0,         0},
    {-protrusionW,            -corner.H, 0},
    {0,                       -corner.H, 0},
    {protrusionW,             -corner.H, 0},
    {protrusionW + corner.W,  0,         0},
    {SCREEN_CENTER_X,         0,         0},
}

local actors = Def.ActorFrame({
    OnCommand = function(self)
        self:xy(SCREEN_CENTER_X, SCREEN_BOTTOM)
    end,
    -- 上下帯
    Def.Quad({
        OnCommand = function(self)
            self:vertalign(bottom)
            self:zoomto(SCREEN_WIDTH+2, 140)
            self:diffuse(bgColor[1], bgColor[2], bgColor[3], 0)
            self:sleep(W_TIMER:GameStartSleep())
            self:linear(0.2)
            self:diffuse(bgColor)
            self:diffusebottomedge(0, 0, 0, bgColor[4]*2/3)
            self:zoomto(SCREEN_WIDTH+2, 50)
        end,
    }),
    -- 曲名部分だけ出っ張りを作る
    Def.ActorFrame({
        OnCommand = function(self)
            self:y(-50)
            self:diffusealpha(0)
            self:sleep(W_TIMER:GameStartSleep())
            self:linear(0.2)
            self:diffusealpha(1)
        end,
        Def.Quad({
            OnCommand = function(self)
                self:vertalign(bottom)
                self:zoomto(protrusionW*2, corner.H)
                self:diffuse(bgColor)
            end,
        }),
        Def.ActorMultiVertex({
            OnCommand = function(self)
                local col = bgColor
                self:SetDrawState({Mode = 'DrawMode_Triangles'})
                self:SetVertices( 1, {
                    {{-protrusionW, 0, 0}, col},
                    {region[3], col},
                    {region[2], col},
                } )
            end,
        }),
        Def.ActorMultiVertex({
            OnCommand = function(self)
                local col = bgColor
                self:SetDrawState({Mode = 'DrawMode_Triangles'})
                self:SetVertices( 1, {
                    {{protrusionW, 0, 0}, col},
                    {region[5], col},
                    {region[6], col},
                } )
            end,
        }),
        -- BPMと曲名の間の線
        Def.Quad({
            OnCommand = function(self)
                self:y(28)
                self:zoomto(0, 1)
                self:diffuse(color('#999999ff'))
                self:sleep(1.5+0.5)
                self:linear(0.2)
                self:zoomto(240, 1)
            end,
        }),
        -- フレームの境界線（下）
        Def.ActorMultiVertex({
            OnCommand = function(self)
                local col = color('#ffffff00')
                local p1Color = GAMESTATE:IsPlayerEnabled(PLAYER_1) and W_DIF:Color(W:Steps(PLAYER_1):GetDifficulty()) or color('#999999ff')
                local p2Color = GAMESTATE:IsPlayerEnabled(PLAYER_2) and W_DIF:Color(W:Steps(PLAYER_2):GetDifficulty()) or color('#999999ff')
                self:SetDrawState({Mode = 'DrawMode_LineStrip'})
                self:SetLineWidth(1)
                self:SetVertices( 1, {
                    {{region[4][1]-1, region[4][2], region[4][3]}, col},
                    {region[4], col},
                    {region[4], col},
                    {region[4], col},
                    {region[4], col},
                    {region[4], col},
                    {{region[4][1]+1, region[4][2], region[4][3]}, col},
                })
                self:sleep(1.5+0.2)
                self:linear(0.2)
                col[4] = 1.0
                self:SetVertices( 1, {
                    {region[3], col},
                    {region[3], col},
                    {region[3], col},
                    {region[4], col},
                    {region[5], col},
                    {region[5], col},
                    {region[5], col},
                })
                self:linear(0.2)
                self:SetVertices( 1, {
                    {region[2], col},
                    {region[2], col},
                    {region[3], col},
                    {region[4], col},
                    {region[5], col},
                    {region[6], col},
                    {region[6], col},
                })
                col = color('#999999ff')
                self:linear(0.2)
                self:SetVertices( 1, {
                    {region[1], p1Color},
                    {region[2], col},
                    {region[3], col},
                    {region[4], col},
                    {region[5], col},
                    {region[6], col},
                    {region[7], p2Color},
                })
            end,
        }),
    }),
    -- 曲名（フレーム上に表示）
    LoadActor(W:Object('song title'), {
        IsCenter    = true,
        IsSmallText = true,
        Width       = 330,
        MainZoom    = 0.85,
        Stroke      = color('0, 0, 0, '..(bgColor[4] < 0.5 and (0.5 - bgColor[4]) or 0)),
    })..{
        OnCommand = function(self)
            self:playcommand('ChangedSong', {Song = W:Song()})
            self:y(-46)
            self:diffusealpha(0)
            self:zoom(0.65)
            self:sleep(math.min(math.max(W_TIMER:GameStartSleep(), 0.2), 2.0))
            self:sleep(0.4)
            self:linear(0.05)
            self:diffusealpha(1)
        end,
        ChangedGameplaySongMessageCommand = function(self, params)
            self:playcommand('ChangedSong', {Song = params.Song})
        end,
    },
})

for pn in ivalues(PlayerNumber) do
    if GAMESTATE:IsPlayerEnabled(pn) then
        -- スコア
        actors[#actors+1] = Def.ActorFrame({
            InitCommand = function(self)
                self:x(-SCREEN_CENTER_X)
            end,
            LoadActor('score', pn)..{
                InitCommand = function(self)
                    local x = SCREEN_CENTER_X + (190 + 0.2 * math.max(SCREEN_WIDTH-640, 0)) * (W:IsP1(pn) and -1 or 1)
                    self:zoom(math.min(SCREEN_WIDTH/854, 1.0))
                    self:xy(x, -20)
                end,
            },
            -- BPM
            LoadActor(
                'bpm',
                pn,
                color('0, 0, 0, '..(bgColor[4] < 0.5 and (0.5 - bgColor[4]) or 0))
            )..{
                InitCommand = function(self)
                    local x = SCREEN_CENTER_X + 70 * (W:IsP1(pn) and -1 or 1)
                    self:xy(x, -12)
                end,
            },
            -- 難易度
            LoadActor('difficulty', pn)..{
                InitCommand = function(self)
                    local x = W:IsP1(pn) and SCREEN_LEFT or SCREEN_RIGHT
                    self:xy(x, -40)
                end,
            },
            -- -- オプション
            -- LoadFont('Common Small Normal')..{
            --     Name = 'Option',
            --     Text = W_PLAYER:GameplayMods(pn),
            --     OnCommand = function(self)
            --         local x = W:IsP1(pn) and SCREEN_LEFT + 150 or SCREEN_RIGHT - 150
            --         self:xy(x, -40)
            --         self:horizalign(W:IsP1(pn) and left or right)
            --         self:diffuse(1, 1, 1, 1)
            --         self:strokecolor(Color('Outline'))
            --         self:zoom(0.45)
            --         self:maxwidth((SCREEN_CENTER_X-180)/0.45)
            --     end,
            -- },
        })
    end
end

return actors