local chara, stageType = ...
local actors = Def.ActorFrame({})

if not chara then return actors end

local dancerSpeed = W_OPTION:ReadPref('DancerSpeed', 'Default') -- or SyncBPM
local dancerShadow = W_OPTION:ReadPref('DancerShadow', 'Off')
local function DancerSpeed(bpm)
    return (dancerSpeed == 'Default') and 1.0*math.min(math.max(120.0 + ((bpm > 120) and (bpm-120.0)/2 or (bpm-120.0)*4/5), 15), 220) or bpm
end

local dir   = chara:GetCharacterDir()
local model = chara:GetModelPath()
local ini = IniFile.ReadFile(dir..'/character.ini')
local zoomSize = ini and ini.Size or 1
local shadowPath = '/Characters/DanceRepo/Shadow/'
local shadowModelPath = shadowPath..(zoomSize <= 0.5 and 'Model_Small.txt' or 'Model.txt')

local fileList = FILEMAN:GetDirListing(dir..'Dance/', false, true)
local bone = fileList[math.ceil(math.random(#fileList))]
local tmpName = split('/', bone)
local boneName = tmpName[#tmpName]
tmpName = split('%.', boneName)
tmpName[#tmpName] = nil
boneName = join('%.', tmpName)
tmpName = nil

-- classicの方が2パターン以上モーション設定するのでこっちで数を合わせるようにマッピングする
local target = {
    Rest   = 1,
    WarmUp = 1,
    Anime1 = 1,
    Anime2 = 1,
    Anime3 = 1,
    Anime4 = 1,
}
local currentAnime = 1
local currentBpm = 120.000

local ChangeDancerBone = function(self, params, i)
    self:finishtweening()
    if not params.Bone or target[params.Bone] == i then
        currentAnime = i
        self:visible(true)
        self:rate(DancerSpeed(currentBpm)/120.0)
    else
        self:visible(false)
        self:rate(0)
        self:position(0)
    end
end
local function ShadowActor(i)
    if dancerShadow == 'Off' or not FILEMAN:DoesFileExist(shadowModelPath) then
        return Def.Actor({})
    end
    return Def.Model({
        Meshes = shadowModelPath,
        Materials = shadowModelPath,
        Bones = shadowPath..'/Dance/'..boneName,
        InitCommand = function(self)
            self:cullmode('CullMode_None')
            self:zbias(1)
            self:visible(i==1 and zoomSize > 0)
        end,
        ChangeDancerBoneCommand = function(self, params)
            ChangeDancerBone(self, params, i)
        end,
    })
end
for i=1, 1 do
    actors[#actors+1] = Def.ActorFrame({
        ShadowActor(i),
        Def.Model({
            Meshes = model,
            Materials = model,
            Bones = bone,
            InitCommand = function(self)
                self:cullmode('CullMode_None')
                self:zoom(zoomSize)
                self:zbias(2)
                self:visible(i==1)
            end,
            ChangeDancerBoneCommand = function(self, params)
                ChangeDancerBone(self, params, i)
            end,
        }),
        OnCommand = function(self)
            self:GetParent():GetParent():playcommand('Zoom', {Stage = 1.0, Character = 1.17})
        end,
        ChangedBpmCommand = function(self, params)
            if currentAnime == i then
                currentBpm = params.Bpm
                self:playcommand('ChangeDancerBone', {})
            end
        end,
    })
end
return actors