-- 背景ダンサー
local screenName = Var('LoadingScreen')

if not ResetCamera then
    ResetCamera = GLOBAL:LocalResetCamera()
end
if not HasVideo then
    HasVideo = GLOBAL:LocalHasVideo()
end

local target = {}
-- プレイヤーが有効な時のみ表示対象（デモはジャケットで隠れてしまうので1体）
target[#target+1] = (W_PLAYER:IsEnabled(PLAYER_1) or GAMESTATE:IsDemonstration()) and PLAYER_1 or nil
if not GAMESTATE:IsDemonstration() then
    target[#target+1] = W_PLAYER:IsEnabled(PLAYER_2) and PLAYER_2 or nil
end
-- ここでtargetの要素数を増やすと3体以上表示可能（キャラはランダム）
-- target[#target+1] = 'Add1'

local dancerIds = {}
for p, id in pairs(target) do
    local chara = (GAMESTATE:IsDemonstration() or p > #PlayerNumber)
                    and CHARMAN:GetRandomCharacter()
                    or ((id == PLAYER_1 or id == PLAYER_2) and GAMESTATE:GetCharacter(id) or CHARMAN:GetRandomCharacter()) or nil
    local charaType = W_DANCER:CharaType(chara)
    if charaType then
        dancerIds[#dancerIds+1] = (id == PLAYER_1 or id == PLAYER_2) and id or (chara and chara:GetCharacterID()) or nil
    end
end

local stage

-- ステージが有効なときのデフォルトカメラ
local fallbackCameras = {
    InitialTween = function()
        ResetCamera()
    end,
    [1] = function(self)
        ResetCamera()
        Camera:rotationx(30):spin():effectmagnitude(0,10,0)
    end,
    [2] = function(self)
        ResetCamera()
        Camera:rotationy(45):rotationx(20):rotationz(-30)
    end,
    [3] = function(self)
        ResetCamera()
        Camera:rotationy(140):rotationz(10):rotationx(-10)
    end,
    [4] = function(self)
        ResetCamera()
        Camera:rotationy(210):rotationx(25)
    end,
    [5] = function(self)
        ResetCamera()
        Camera:rotationx(70):z(WideScale(190,290))
    end,
}
-- キャラクターのみの時のカメラ
local noStageCameras = {
    InitialTween = function()
        ResetCamera()
    end,
    [1] = function(self)
        ResetCamera()
        Camera:rotationx(30):spin():effectmagnitude(0,10,0)
    end,
    [2] = function(self)
        ResetCamera()
        Camera:rotationy(45):rotationx(20):rotationz(-30):spin():effectmagnitude(0,-2,0)
    end,
    [3] = function(self)
        ResetCamera()
        Camera:rotationy(140):rotationz(10):rotationx(-10):spin():effectmagnitude(0,2,0)
    end,
    [4] = function(self)
        ResetCamera()
        Camera:rotationy(210):rotationx(25)
    end,
}
local cameraCommands = {}
local totalCameraAngles = 0
local cameraAngle = 0
local changedCameraAngleCount = 0
local lastCameraSeconds = -9999
local waitCameraSeconds = 0

local function StageUpdate(_self)
    local parent = _self:GetParent()
    if parent:GetChild('Stage') then
        parent:RemoveChild('Stage')
    end
    -- Group.luaのStage対応
    local stageName = YA_LIB.GROUP:Custom(W:Song(), 'Stage') or W_OPTION:ReadPref('Stage', 'Off')
    -- Project OutFoxクレジットの場合はステージOffに設定
    if screenName == 'ScreenCreditsGameplay' then
        stageName = 'Off'
    end
    -- 今後の拡張用: ステージ名>キャラクター名 のような区切り方をできるようにする
    local matchStageName = string.match(stageName, '[^>|:;]+')
    local hasStage = (matchStageName ~= 'Off')
    parent:fov(hasStage and 90 or 60)
    stage = W_DANCER:GetStage(matchStageName or '', 'Off')
    if stage then
        if stage.Type == 'outfox' and not W:IsOF() then
            -- OFじゃないのにOF用のカメラの場合
            cameraCommands = fallbackCameras
            totalCameraAngles = #cameraCommands
        elseif FILEMAN:DoesFileExist(stage.Path..'Cameras.lua') then
            local cameraFile = LoadActor(stage.Path..'Cameras')
            if stage.Type == 'enciso' then
                for message,command in pairs(cameraFile) do
                    local r, _, pt = string.find(string.lower(message), '^camera(%d+)message')
                    if r then
                        cameraCommands[tonumber(pt)] = command
                        totalCameraAngles = totalCameraAngles+1
                    end
                end
            else
                cameraCommands = cameraFile
                totalCameraAngles = #cameraCommands
            end
        else
            cameraCommands = fallbackCameras
            totalCameraAngles = #cameraCommands
        end
        parent:playcommand('HideBG')
    else
        -- ステージ無しの時のカメラワーク
        cameraCommands = noStageCameras
        totalCameraAngles = #cameraCommands
    end
    local tempPath = THEME:GetCurrentThemeDirectory()..'./Other/StageTempActor.lua'
    local tempStage = ''
    tempStage = tempStage.. "return Def.ActorFrame({\n"
    tempStage = tempStage.. "  Name = 'Stage',\n"
    tempStage = tempStage.. "  InitCommand = function(self) self:playcommand('On') end,\n"
    if stage then
        tempStage = string.format("%s  W_DANCER:Stage('%s'),\n", tempStage, stage.Name)
    end
    tempStage = string.format("%s  W_DANCER:Character('%s', '%s'),\n", tempStage, join(':', dancerIds), stage and stage.Type or '')
    if stage then
        tempStage = string.format("%s  W_DANCER:Stage('%s', true),\n", tempStage, stage.Name)
    end
    tempStage = tempStage.. "})\n"

    local file = RageFileUtil:CreateRageFile()
    file:Open(tempPath, 2)
    file:Write(tempStage)
    file:Close()
    file:destroy()
    parent:AddChildFromPath(tempPath)
    local zoom = 1.0*SCREEN_HEIGHT/720.0    -- Encisoシステムは1080と思いきや720でちゃんと動いている謎
    parent:GetParent():zoom(zoom)
        :xy(SCREEN_CENTER_X/2*zoom, SCREEN_CENTER_Y/2*zoom):z(WideScale(300, 400)/2*zoom)
end

local isFirstCamAngle = true
local isCameraStop = false
local effx, effy, effz = 0, 0, 0

return Def.ActorFrame({
    Def.ActorFrame({
        InitCommand=function(self)
            Camera = self
            self:playcommand('ChangedGameplaySongResetCamera')
        end,
        ChangedGameplaySongMessageCommand = function(self)
            StageUpdate(self:GetChild('Stage'))
            self:playcommand('ChangedGameplaySongResetCamera')
        end,
        ChangedGameplaySongResetCameraCommand = function(self)
            if cameraCommands.InitialTween then
                cameraCommands.InitialTween()
            else
                fallbackCameras.InitialTween()
            end
            changedCameraAngleCount = 0
        end,
        HereWeGoMessageCommand = function(self)
            self:finishtweening()
            self:playcommand('ChangeDancerBone', {
                Bone = 'WarmUp',
            })
        end,
        FirstBeatMessageCommand = function(self)
            isFirstCamAngle = false
            self:finishtweening()
            self:playcommand('While32Beat')
        end,
        -- ダンサーのアニメーション切り替え(32拍毎)
        While32BeatMessageCommand = function(self)
            self:playcommand('ChangeDancerBone', {
                Bone = 'Anime'..math.ceil(math.random(4)),
            })
        end,
        ChangedCameraAngleCommand = function(self)
            if not Camera then Camera = self end
            if isFirstCamAngle or totalCameraAngles <= 0 then return end
            -- カメラアングルを切り替え(16拍毎)
            local r = math.random(totalCameraAngles)-1
            -- 同じアングルが連続で来ないようにする
            cameraAngle = (cameraAngle ~= r) and r or ((r+1)%totalCameraAngles)
            cameraCommands[cameraAngle+1]()
            changedCameraAngleCount = changedCameraAngleCount + 1
            waitCameraSeconds = WaitTime and WaitTime[cameraAngle+1] or 7
            effx, effy, effz = self:geteffectmagnitude()
            isCameraStop = false
        end,
        ScreenUpdateMessageCommand = function(self, params)
            if not Camera then Camera = self end
            if isFirstCamAngle or totalCameraAngles <= 0 then return end
            local stop = (GAMESTATE:GetNumPlayersEnabled() == 1)
                            and params.PnStop[GAMESTATE:GetMasterPlayerNumber()]
                            or params.Stop
            -- BPM0の時はカメラ速度を下げる
            if stop and not isCameraStop then
                self:hurrytweening(5.0)
                self:effectmagnitude(effx*0.2, effy*0.2, effz*0.2)
                isCameraStop = true
            elseif not stop and isCameraStop then
                self:hurrytweening(0.2)
                self:effectmagnitude(effx, effy, effz)
                isCameraStop = false
            end
            -- カメラアングルを切り替え
            if params.Sec - lastCameraSeconds >= waitCameraSeconds then
                isCameraStop = false
                self:finishtweening()
                self:playcommand('ChangedCameraAngle')
                lastCameraSeconds = params.Sec
            end
        end,
        Def.Actor({
            OnCommand = function(self)
                self.BgActor = SCREENMAN:GetTopScreen():GetChild("SongBackground"):GetChild("")
            end,
            HideBGMessageCommand = function(self)
                if self.BgActor then
                    self.BgActor:visible(false)
                end
            end,
            ShowBGMessageCommand = function(self)
                if self.BgActor then
                    self.BgActor:visible(true)
                end
            end,
        }),
        Def.ActorFrame({
            Name = 'Stage',
            -- StageのluaはOtherフォルダにリアルタイムで書き出してAddChildFromPath経由で使用
        }),
    }),
})
