local width = 160
local height = 28
local mpn = GAMESTATE:GetMasterPlayerNumber()
local offset = 0

local timer = -1
local function update(self, delta)
    timer = timer + delta
    if math.floor(math.max(timer, 0)) ~= offset then
        local trail = GAMESTATE:GetCurrentTrail(mpn)
        local tEntry = trail and trail:GetTrailEntries() or {}
        if #tEntry > 4 then
            offset = math.floor(timer)%(#tEntry)
            if offset == 0 then timer = -1 end
            self:playcommand('Scroll')
        end
    end
end

local function SongListActor(index)
    return Def.ActorFrame({
        Name = 'Song'..index,
        OnCommand = function(self)
            self:ztest(true)
            self:y((index-1)*height)
            self:visible(false)
            self:playcommand('CurrentTrailP1Changed')
        end,
        CurrentTrailP1ChangedMessageCommand = function(self)
            local trail = GAMESTATE:GetCurrentTrail(mpn)
            local tEntry = trail and trail:GetTrailEntries() or {}
            local i = (#tEntry > 4) and (index + offset - 1) % #tEntry + 1 or index
            if tEntry and tEntry[i] then
                local course = GAMESTATE:GetCurrentCourse()
                self:visible(true)
                self:playcommand('ChangedTrail', {
                    Song   = tEntry[i]:GetSong(),
                    Steps  = tEntry[i]:GetSteps(),
                    Course = course,
                    Index  = i,
                    Trail  = trail,
                })
            else
                self:visible(false)
            end
        end,
        CurrentTrailP2ChangedMessageCommand = function(self)
            self:playcommand('CurrentTrailP1Changed')
        end,
        ScrollCommand = function(self)
            self:finishtweening()
            self:playcommand('CurrentTrailP1Changed')
            self:addy(height)
            self:linear(0.2)
            self:addy(-height)
        end,
        Def.Quad({
            OnCommand = function(self)
                self:zoomto(width*2, 25)
            end,
            ChangedTrailCommand = function(self, params)
                if params.Steps then
                    self:diffuse(BoostColor(W_DIF:Color(params.Steps:GetDifficulty()), 0.25))
                end
            end,
        }),
        Def.Quad({
            OnCommand = function(self)
                self:x(-width)
                self:horizalign(left)
                self:zoomto(width*2-90, 25)
                self:diffuse(0, 0, 0, 0.5)
                self:faderight(0.33)
            end,
        }),
        -- バナー表示はキャッシュでも重いので却下
        -- W_GRAPHICS:FixedRatio(64, 20, THEME:GetPathG('common', 'fallback banner'))..{
        --     OnCommand = function(self)
        --         self:x(-60)
        --     end,
        --     ChangedTrailCommand = function(self, params)
        --         if params.Song then
        --             self:playcommand('UpdateParams', {
        --                 File = W_GRAPHICS:SongBanner(params.Song),
        --             })
        --         end
        --     end,
        -- },
        -- ステージ数
        LoadFont('Common Bold')..{
            OnCommand = function(self)
                self:xy(-width+15, -3)
                -- self:xy(-135, -3)
                -- self:horizalign(left)
                self:zoom(0.75)
                self:maxwidth(25 / 0.75)
            end,
            ChangedTrailCommand = function(self, params)
                if params.Song then
                    self:diffuse(1, 1, 1, 1)
                    self:settext(params.Index)
                end
            end,
        },
        LoadFont('Common Small Normal')..{
            OnCommand = function(self)
                self:xy(-width+28, 7)
                self:horizalign(right)
                self:zoom(0.35)
            end,
            ChangedTrailCommand = function(self, params)
                if params.Song then
                    local text = 'th'
                    if math.floor(params.Index%100/10) ~= 1 then
                        if params.Index%10 == 1 then
                            text = 'st'
                        elseif params.Index%10 == 2 then
                            text = 'nd'
                        elseif params.Index%10 == 3 then
                            text = 'rd'
                        end
                    end
                    self:diffuse(1, 1, 1, 1)
                    self:settext(text)
                end
            end,
        },
        -- 難易度数値
        LoadFont('Common Bold')..{
            OnCommand = function(self)
                self:xy(width-5, 0)
                self:horizalign(right)
                self:zoom(0.95)
                self:maxwidth(30 / 0.95)
            end,
            ChangedTrailCommand = function(self, params)
                if params.Steps then
                    self:diffuse(BoostColor(W_DIF:Color(params.Steps:GetDifficulty()), 1))
                    self:settext(params.Trail:IsSecret() and '?' or params.Steps:GetMeter())
                end
            end,
        },
        -- 難易度名
        LoadFont('Common Small Bold')..{
            OnCommand = function(self)
                self:xy(width-80, -5)
                self:zoom(0.3)
                self:maxwidth(45 / 0.3)
                self:horizalign(left)
            end,
            ChangedTrailCommand = function(self, params)
                if params.Steps then
                    self:diffuse(1, 1, 1, 1)
                    self:settext(W_DIF:Name(params.Steps:GetDifficulty()))
                end
            end,
        },
        -- METERTYPE
        LoadFont('Common Small Normal')..{
            OnCommand = function(self)
                self:xy(width-80, 5)
                self:zoom(0.3)
                self:maxwidth(45 / 0.3)
                self:horizalign(left)
            end,
            ChangedTrailCommand = function(self, params)
                if params.Song then
                    self:diffuse(1, 1, 1, 1)
                    self:settext(YA_GROUP:MeterType(params.Song))
                end
            end,
        },
        -- 曲名
        LoadFont('Common Normal')..{
            OnCommand = function(self)
                self:xy(-width+35, -4)
                self:horizalign(left)
                self:strokecolor(Color('Outline'))
                self:zoom(0.5)
                self:maxwidth((width*2-120) / 0.5)
            end,
            ChangedTrailCommand = function(self, params)
                if params.Song and not params.Trail:IsSecret() then
                    local song = params.Song
                    self:diffuse(YA_GROUP:MenuColor(song))
                    self:settext(song:GetDisplayFullTitle())
                else
                    self:diffuse(1, 1, 1, 1)
                    self:settext('???')
                end
            end,
        },
        -- グループ名
        LoadFont('Common Small Normal')..{
            OnCommand = function(self)
                self:xy(-width+35, 6)
                self:horizalign(left)
                self:zoom(0.35)
                self:maxwidth((width*2-120) / 0.35)
            end,
            ChangedTrailCommand = function(self, params)
                if params.Song then
                    local group = params.Song:GetGroupName()
                    self:diffuse(YA_GROUP:GroupColor(group))
                    self:settext(YA_GROUP:GroupName(group))
                else
                    self:diffuse(1, 1, 1, 1)
                    self:settext('RANDOM')
                end
            end,
        },
    })
end

return Def.ActorFrame({
    OnCommand = function(self)
        self:SetUpdateFunction(update)
    end,
    CurrentCourseChangedMessageCommand = function(self)
        timer  = -1
        offset = 0
    end,
    LoadActor(W:Object('style'), 100, true)..{
        InitCommand = function(self)
            self:y(-height)
        end,
        OnCommand=function(self)
            self:playcommand('CurrentTrailP1Changed')
        end,
        CurrentTrailP1ChangedMessageCommand = function(self)
            local trail = GAMESTATE:GetCurrentTrail(mpn)
            self:playcommand('CallbackSteps', {
                Song      = nil,
                Steps     = nil,
                Player    = nil,
                StepsType = trail:GetStepsType(),
            })
        end,
        CurrentTrailP2ChangedMessageCommand = function(self)
            self:playcommand('CurrentTrailP1Changed')
        end,
    },
    Def.ActorFrame({
        Def.Quad({
            OnCommand=function(self)
                self:zoomto(SCREEN_WIDTH, 25)
                self:y(-height)
                self:clearzbuffer(true)
                self:zwrite(true)
                self:blend('BlendMode_NoEffect')
            end,
        }),
        Def.Quad({
            OnCommand=function(self)
                self:zoomto(SCREEN_WIDTH, 25)
                self:y(height*4)
                self:zwrite(true)
                self:blend('BlendMode_NoEffect')
            end,
        }),
        SongListActor(0),
        SongListActor(1),
        SongListActor(2),
        SongListActor(3),
        SongListActor(4),
    }),
})
