local pn = ...
local size = 112
local textR = 108    -- テキストを表示するレーダー中央からの位置
local radarColor = W_PLAYER:Color(pn)
local defaultVertices = {}
for i=1, 7 do
    defaultVertices[#defaultVertices+1] = {{0, 0, 0}, radarColor}
end
local categories = {
    'RadarCategory_Stream',
    'RadarCategory_Voltage',
    'RadarCategory_Air',
    'RadarCategory_Freeze',
    'RadarCategory_Chaos',
}
local categoryNames = {'STR','VOL','AIR','FRZ','CHA'}

local textActor = Def.ActorFrame({})
for r = 1, #categories do
    textActor[#textActor+1] = LoadFont('Common Small Normal')..{
        InitCommand = function(self)
            self:diffuse(color('#ffffffe0'))
            self:strokecolor(Color('Outline'))
            self:x(-0.5 * textR * math.sin(math.pi*2*(r-1)/5))
            self:y(-0.5 * textR * math.cos(math.pi*2*(r-1)/5))
            self:zoom(0.4)
            self:settext('')
        end,
        ChangedRadarMessageCommand = function(self, params)
            if params.Song then
                self:settextf('%s\n%d', categoryNames[r], math.floor(2*params['Radar'][categories[r]]))
            else
                self:settextf('%s\n', categoryNames[r])
            end
            self:AddAttribute(0, {Length = 3, Diffuse = color('1.0, 1.0, 1.0, 0.75')})
        end,
    }
end

return Def.ActorFrame({
    InitCommand = function(self)
        self:visible(W_PLAYER:IsEnabled(pn))
    end,
    -- 処理
    UpdateStepsMessageCommand = function(self, params)
        self:visible(W_PLAYER:IsEnabled(pn))
        if params.Player and params.Player ~= pn then
            return
        end
        local radar = W_PLAYER:Radar(pn)
        local vertices = {}
        vertices[#vertices+1] = {{0, 0, 0}, color('#ffffff80')}
        for r = 1, #categories + 1 do
            local val = params.Song and radar[categories[(r-1)%5 + 1]] * 0.95 + 5 or 0
            if val > 100 then
                val = (val-100) * 2 +100
            end
            vertices[#vertices+1] = {
                {
                    -0.005 * size * val * math.sin(math.pi*2*(r-1)/5),
                    -0.005 * size * val * math.cos(math.pi*2*(r-1)/5),
                    0,
                },
                radarColor,
            }
        end
        self:playcommand('ChangedRadar', {
            Song     = params.Song,
            Player   = params.Player,
            Vertices = vertices,
            Radar    = radar,
        })
    end,
    -- 背景
    Def.ActorMultiVertex({
        InitCommand = function(self)
            local vertices = {}
            vertices[#vertices+1] = {{0, 0, 0}, {radarColor[1], radarColor[2], radarColor[3], 0.5}}
            for r = 1, #categories + 1 do
                vertices[#vertices+1] = {
                    {-0.5 * size * math.sin(math.pi*2*(r-1)/5), -0.5 * size * math.cos(math.pi*2*(r-1)/5), 0},
                    color('#002000a0'),
                }
            end
            self:SetVertices(vertices)
            self:SetDrawState({Mode = 'DrawMode_Fan'})
        end,
    }),
    Def.ActorMultiVertex({
        InitCommand = function(self)
            local vertices = {}
            vertices[#vertices+1] = {{0, 0, 0}, color('#00000080')}
            for r = 1, #categories + 1 do
                vertices[#vertices+1] = {
                    {-0.27 * size * math.sin(math.pi*2*(r-1)/5), -0.27 * size * math.cos(math.pi*2*(r-1)/5), 0},
                    color('#8aaa8a40'),
                }
            end
            self:SetVertices(vertices)
            self:SetDrawState({Mode = 'DrawMode_Fan'})
        end,
    }),
    -- レーダー
    Def.ActorMultiVertex({
        InitCommand = function(self)
            self:blend('BlendMode_Add')
            self:SetVertices(defaultVertices)
            self:SetDrawState({ Mode = 'DrawMode_Fan' })
        end,
        ChangedRadarMessageCommand = function(self, params)
            self:stoptweening()
            self:linear(0.2)
            self:SetVertices(params.Vertices)
        end,
    }),
    -- 境界線
    Def.ActorMultiVertex({
        InitCommand = function(self)
            self:SetVertices(defaultVertices)
            self:SetLineWidth(0)
            self:SetDrawState({Mode = 'DrawMode_LineStrip', First= 2, Num= 6})
        end,
        ChangedRadarMessageCommand = function(self, params)
            self:stoptweening()
            self:linear(0.2)
            self:SetVertices(params.Vertices)
        end,
    }),
    -- 数値
    textActor,
    -- Autogen
    LoadActor('autogen')..{
        InitCommand = function(self)
            self:diffusealpha(0)
            self:xy((W:IsP1(pn) and -1 or 1)*35, -40)
        end,
        UpdateStepsMessageCommand = function(self, params)
            if not params.Song then
                self:diffusealpha(0)
                return
            end
            if params.Player == pn then
                self:stoptweening()
                self:diffusealpha(0)
                if params.Steps and params.Steps:IsAutogen() then
                    self:zoom(1.5)
                    self:linear(0.1)
                    self:zoom(1.0)
                    self:diffusealpha(1)
                    self:glowblink()
                end
            end
        end,
    },
})