local player, width = ...
width = width or 170
local lists = Def.ActorFrame({
    InitCommand = function(self)
        self:visible(W_PLAYER:IsEnabled(player))
    end,
    UpdateStepsMessageCommand = function(self, params)
        self:visible(W_PLAYER:IsEnabled(player))
    end,
    -- 漢字表記
    LoadActor('kanji')..{
        Text = '',
        InitCommand = function(self)
            self:diffuse(Color('White'))
            self:x(width/2+4)
            self:y(4)
            self:animate(false)
        end,
        UpdateStepsMessageCommand = function(self, params)
            if not params.Song then
                self:diffuse(Color('White'))
                self:setstate(6)
                return
            end
            if params.Player == player then
                self:diffuse(W_DIF:Color(params.Difficulty))
                self:setstate(Enum.Reverse(Difficulty)[params.Difficulty] or 6)
            end
        end,
    },
    -- フレーム
    LoadActor('frame')..{
        InitCommand = function(self)
            self:diffuse(Color('White'))
        end,
        UpdateStepsMessageCommand = function(self, params)
            if not params.Song then
                self:diffuse(Color('White'))
                return
            end
            if params.Player == player then
                self:diffuse(W_DIF:Color(params.Difficulty))
            end
        end,
    },
    -- 難易度
    LoadFont('Common Small Bold')..{
        Text = '',
        InitCommand = function(self)
            self:diffuse(Color('White'))
            --self:strokecolor(Color('Outline'))
            self:vertalign(top)
            self:horizalign(left)
            self:x(-width/2-2)
            self:y(-14)
            --self:skewx(-0.2)
            self:zoom(0.68)
            self:maxwidth((width-50)/0.68-15)
        end,
        UpdateStepsMessageCommand = function(self, params)
            if not params.Song then
                self:diffuse(Color('White'))
                self:settext('GROUP')
                return
            end
            if params.Player == player then
                local text = ''
                if params.Difficulty == 'Difficulty_Edit' then
                    text = (params.StepName ~= '') and params.StepName or W_DIF:Name(params.Difficulty)
                elseif params.Difficulty then
                    text = string.upper(W_DIF:Name(params.Difficulty))
                end
                self:diffuse(W_DIF:Color(params.Difficulty))
                self:settext(text)
            end
        end,
    },
    -- 数字
    LoadFont('Common Bold')..{
        Text = '',
        InitCommand = function(self)
            self:diffuse(Color('White'))
            --self:strokecolor(Color('Outline'))
            self:vertalign(top)
            self:horizalign(right)
            self:x(width/2-15)
            self:y(-14)
            --self:skewx(-0.2)
            self:zoom(1.4)
            self:maxwidth(45/1.4)
        end,
        UpdateStepsMessageCommand = function(self, params)
            if not params.Song then
                self:diffuse(Color('White'))
                self:settext('--')
                return
            end
            if params.Player == player and params.Steps then
                self:diffuse(W_DIF:Color(params.Difficulty))
                self:settext(params.Steps:GetMeter())
            end
        end,
    },
    -- METERTYPE
    LoadFont('Common Small Normal')..{
        Text = '',
        InitCommand = function(self)
            self:diffuse(Color('White'))
            self:diffusealpha(0.8)
            self:vertalign(top)
            self:horizalign(right)
            self:x(width/2+10)
            self:y(13)
            self:zoom(0.35)
            self:maxwidth(65/0.35)
        end,
        ChangedSongMessageCommand = function(self, params)
            if not params.Song then
                self:settext('')
                return
            end
            self:settext(YA_GROUP:MeterType(params.Song))
        end,
    },
    -- ハイスコア
    LoadFont('waiei2 Score Numbers')..{
        Text = '',
        InitCommand = function(self)
            self:diffuse(Color('White'))
            self:diffusealpha(0.8)
            self:horizalign(right)
            self:x(10)
            self:y(9)
            self:zoom(0.8)
            self:maxwidth(130/0.8)
        end,
        UpdateStepsMessageCommand = function(self, params)
            if not params.Song then
                self:diffusealpha(0.5)
                self:settext('0.00%')
                return
            end
            if params.Player == player and params.Steps then
                local profile = PROFILEMAN:GetProfile(player)
                local highscores = profile and profile:GetHighScoreList(params.Song, params.Steps):GetHighScores() or nil
                local percent = (#highscores > 0) and highscores[1]:GetPercentDP() or 0
                if percent <= 0 then
                    self:diffusealpha(0.5)
                elseif percent >= 100 then
                    self:diffusealpha(1)
                else
                    self:diffusealpha(1)
                end
                self:settext(W_STRING:DP(percent))
            end
        end,
    },
})

return lists