local w=192
local h=170

-- IDに一意な値を設定する手段がないので
-- Graphics/Objectsではなく、Wheel用に用意したオブジェクトを使用する
local Banner = LoadActor(W:Object('banner'), {
    Course = nil,
    Size   = w,
    Ratio  = 256/80,
    Cache  = -1,
    Rate   = 0.333333,
})

local TopBorder = LoadActor(W:Object('radius'), {
    Width  = w,
    Height = 16,
    Radius = 8,
    Color  = {1, 1, 1, 1},
    Cell   = {1, 1, 1, 0, 0, 0, 0, 0, 0},
})

local MusicLength = LoadActor(W:Object('radius'), {
    Width  = 80,
    Height = 16,
    Radius = 8,
    Color  = {1, 1, 1, 1},
})

-- 背景
local function BgActor()
    return Def.ActorFrame({
        -- 影
        LoadActor(W:Object('boxshadow'), 'wheel', {
            Length = 10,
            Color  = {0.1, 0.1, 0.1, 0.4},
            Center = {0.1, 0.1, 0.1, 0.4},
        })..{
            OnCommand = function(self)
                self:y(6)
                self:playcommand('UpdateBoxShadow', {
                    Id         = 'wheel',
                    Width      = w,
                    Height     = h-8,
                    Wait       = 0.5,
                    AnimeAlpha = true,
                })
            end,
        },
        -- 上部バー
        TopBorder:Actor()..{
            OnCommand = function(self)
                self:y(-h/2 + 8)
            end,
            UpdateWheelChildCommand = function(self, params)
                local course = params.Course
                local color = {1, 1, 1, 0.8}
                if course and course:IsAutogen() then
                    color = {1, 1, 1, 0.8}
                else
                    color = {0, 0.9, 1, 0.8}
                end
                color[4] = params.HasFocus and 1 or 0.75
                self:playcommand('UpdateRadius', {
                    Color = color,
                })
            end,
        },
        -- 背景
        Def.Quad({
            InitCommand = function(self)
                self:zoomto(w, h - 7)
                self:vertalign(top)
                self:y(-h/2 + 7)
                self:diffuse(0.25, 0.25, 0.3, 1)
            end,
        }),
        Def.Quad({
            InitCommand = function(self)
                self:y(-h/2 + 140)
                self:zoomto(w, 1)
                self:diffuse(color('#666666ff'))
            end,
        }),
    })
end

-- バナー
local function BannerActor()
    return Def.ActorFrame({
        InitCommand = function(self)
            self:xy(0, -h/2+38)
        end,
        -- バナー
        Banner:Actor()..{
            UpdateWheelChildCommand=function(self,params)
                self:playcommand('UpdateBanner', {
                    Path = W_GRAPHICS:CourseBanner(params.Course),
                    --Path = params.Course:GetBannerPath(),
                })
            end,
        }
    })
end

-- コース名
local function CourseTitleActor()
    return Def.ActorFrame({
        InitCommand = function(self)
            self:xy(0, -h/2 + 85)
        end,
        -- タイトル
        LoadFont('Common Small Normal')..{
            InitCommand=function(self)
                self:diffuse(color('#d8d8d8ff'))
                self:horizalign(left)
                self:vertalign(top)
                self:xy(-w/2+10, 0)
            end,
            UpdateWheelChildCommand=function(self,params)
                local course = params.Course
                if course then
                    local fullTitle = YA_GROUP:GroupName(course:GetDisplayFullTitle())
                    local match = string.match(fullTitle, '(.+) Random')
                    if match then
                        fullTitle = YA_GROUP:GroupName(match)..' Random'
                    end
                    local textZoom = 0.9
                    self:zoom(textZoom)
                    self:maxwidth((w-20)/textZoom)
                    self:settext(W_STRING:CutWidth(fullTitle, 13))
                    self:diffusealpha(params.HasFocus and 1 or 0.75)
                end
            end,
        },
    })
end

-- 情報
local function CourseTimeActor()
    return Def.ActorFrame({
        InitCommand = function(self)
            self:xy(0, -h/2 + 115)
        end,
        -- 曲数
        LoadFont('Common Small Normal')..{
            InitCommand=function(self)
                local textZoom = 0.75
                self:diffuse(color('#a0a0a0ff'))
                self:horizalign(left)
                self:vertalign(top)
                self:zoom(textZoom)
                self:maxwidth((w/2-20)/textZoom)
                self:xy(-w/2+10, 0)
            end,
            UpdateWheelChildCommand=function(self,params)
                self:diffuse(color('#a0a0a0ff'))
                local course = params.Course
                if course then
                    local count = #course:GetAllTrails()[1]:GetTrailEntries()
                    self:settext(count..' Song'..((count ~= 1) and 's' or ''))
                    self:diffusealpha(params.HasFocus and 1 or 0.5)
                end
            end,
        },
        Def.ActorFrame({
            InitCommand=function(self)
                self:xy(w/2-10-40, 6)
            end,
            MusicLength:Actor()..{
                UpdateWheelChildCommand=function(self, params)
                    local course = params.Course
                    local color = color('0, 0, 0, 0')
                    if course then
                        -- 長さソートで不自然にならないように切り捨て
                        local courseType = course:GetCourseType()
                        if courseType == 'CourseType_Oni' then
                            color = {0.9, 0.3, 0.3, params.HasFocus and 1 or 0.5}
                        elseif courseType == 'CourseType_Endless' then
                            color = {0.2, 0.75, 1.0, params.HasFocus and 1 or 0.5}
                        elseif courseType == 'CourseType_Survival' then
                            color = {0.2, 0.75, 1.0, params.HasFocus and 1 or 0.5}
                        else
                            color = {0.55, 0.65, 0.3, params.HasFocus and 1 or 0.5}
                        end
                    end
                    self:playcommand('UpdateRadius', {
                        Color = color,
                    })
                end,
            },
            LoadFont('Common Small Bold')..{
                InitCommand=function(self)
                    local textZoom = 0.5
                    self:y(-1)
                    self:diffuse(color('#ffffffff'))
                    self:zoom(textZoom)
                    self:maxwidth((w/2-20)/textZoom)
                end,
                UpdateWheelChildCommand=function(self,params)
                    local course = params.Course
                    if course then
                        -- 長さソートで不自然にならないように切り捨て
                        local courseType = course:GetCourseType()
                        self:settext(string.upper(ToEnumShortString(courseType)))
                        self:diffusealpha(params.HasFocus and 1 or 0.5)
                    end
                end,
            },
        }),
    })
end

-- グループ
local function GenreActor()
    return Def.ActorFrame({
        LoadFont('Common Small Normal')..{
            InitCommand = function(self)
                self:horizalign(right)
                self:xy(w/2 - 10, -h/2 + 155)
                self:zoom(0.5)
                self:maxwidth((w-20)/0.5)
                self:diffuse(color('#999999ff'))
                self:settext('')
            end,
            UpdateWheelChildCommand=function(self, params)
                local text = ((params.Course and params.Course:GetGroupName() ~= '')
                                and params.Course:GetGroupName()
                                or 'Course')
                self:settext(W_STRING:CutWidth(text, 22))
            end,
        },
    })
end

return Def.ActorFrame({
    SetCommand=function(self, params)
        -- 多重呼び出しを回避
        if not self.Params then
            self.Params = {Id = math.random(), Course = nil, HasFocus = nil}
            TopBorder:Id('B'..self.Params.Id)
            MusicLength:Id('ML'..self.Params.Id)
        end
        if self.Params and params and params.Course and params.HasFocus ~= nil then
            if params.Course ~= self.Params.Course or params.HasFocus ~= self.Params.HasFocus then
                params.Id = self.Params.Id
                self.Params.Course = params.Course
                self.Params.HasFocus = params.HasFocus
                self:finishtweening()
                self:playcommand('UpdateWheel', params)
            end
        end
    end,
    UpdateWheelCommand = function(self, params)
        if self.Params.Id ~= params.Id then return end
        self:playcommand('UpdateWheelChild', params)
    end,
    -- 背景
    BgActor(),
    -- バナー
    BannerActor(),
    -- コース名
    CourseTitleActor(),
    -- 曲数
    CourseTimeActor(),
    -- グループ
    GenreActor(),
})
