local pn, height = ...

local w = 180
local function TextTemplateActor(textList)
    return Def.ActorFrame({
        LoadFont('Common Small Normal')..{
            Text = textList[1],
            InitCommand = function(self)
                self:diffuse(Color('White'))
                self:zoom(0.35)
                self:horizalign(left)
                self:vertalign(top)
                self:xy(-w/2+10, 0)
            end,
        },
        LoadFont('Common Normal')..{
            Text = textList[2],
            InitCommand = function(self)
                self:diffuse(Color('White'))
                self:zoom(0.6)
                self:horizalign(right)
                self:vertalign(bottom)
                self:xy(w/2-40, 25)
            end,
        },
        LoadFont('Common Small Normal')..{
            Text = textList[3],
            InitCommand = function(self)
                self:diffuse(Color('White'))
                self:zoom(0.35)
                self:horizalign(right)
                self:vertalign(bottom)
                self:xy(w/2-10, 25)
            end,
        },
    })
end

local profile = PROFILEMAN:GetProfile(pn)
if not W_PLAYER:IsEnabled(pn) then
    return Def.ActorFrame({})
end

return Def.ActorFrame({
    OnCommand = function(self)
        local x = SCREEN_CENTER_X - w/2
        self:xy(W:IsP1(pn) and -x or x, -height/2)
    end,
    -- 背景
    Def.Quad({
        OnCommand = function(self)
            local playerColor = W_PLAYER:Color(pn)
            self:diffuse(BoostColor({playerColor[1], playerColor[2], playerColor[3], 0.5}, 0.75))
            self:zoomto(w, height)
            self:y(height)
            self:vertalign(bottom)
            if pn == PLAYER_1 then
                self:faderight(1.0)
            else
                self:fadeleft(1.0)
            end
        end,
    }),
    -- プレイヤー名
    Def.ActorFrame({
        OnCommand = function(self)
            self:y(20)
        end,
        Def.Quad({
            OnCommand = function(self)
                local playerColor = W_PLAYER:Color(pn)
                self:diffuse(BoostColor({playerColor[1], playerColor[2], playerColor[3], 0.75}, 0.75))
                self:zoomto((w-10), 2)
                self:y(50)
                self:blend('BlendMode_Add')
                self:fadeleft(0.5)
                self:faderight(0.5)
            end,
        }),
        LoadFont('Common Small Normal')..{
            Text = W_PLAYER:Name(pn),
            InitCommand = function(self)
                self:zoom(0.66)
                self:maxwidth((w-70)/0.66)
                self:y(35)
            end,
        },
        W_AVATAR:Frame(W_PLAYER:PlayerAvatar(pn), 'Avatar'..pname(pn), {
                Flip = W:IsP1(pn),
                Mini = false,
                Color = BoostColor(W_PLAYER:Color(pn), 0.75),
                Type  = 0,
                RandomNum = W_AVATAR:RandomNum(W_PLAYER:Name(pn)),
        })..{
            InitCommand = function(self)
                self:zoom(0.45)
                self:y(-5)
            end,
        },
    }),
    -- 本日の累計消費カロリー
    TextTemplateActor({
        THEME:GetString('Player', 'CaloriesBurnedToday'),
        commify(string.format('%4.1f', profile:GetCaloriesBurnedToday()), ','),
        'Kcal',
    })..{
        OnCommand = function(self)
            self:xy(0, 90)
        end,
    },
    -- これまでの累計消費カロリー
    TextTemplateActor({
        THEME:GetString('Player', 'TotalCaloriesBurned'),
        commify(string.format('%4.1f', profile:GetTotalCaloriesBurned()), ','),
        'Kcal',
    })..{
        OnCommand = function(self)
            self:xy(0, 140)
        end,
    },
    -- 累計プレイ曲数
    TextTemplateActor({
        THEME:GetString('Player', 'TotalSongsPlayed'),
        commify(string.format('%3d', profile:GetTotalNumSongsPlayed()), ','),
        'Songs',
    })..{
        OnCommand = function(self)
            self:xy(0, 190)
        end,
    },
})

