local screen
local cursor = 1

local optionAll = dofile(THEME:GetPathO('', 'Options.lua'))
local option = {Menu = {}, OptionList = {}}
option.OptionList = optionAll.OptionList
-- オプションを読み込んで変数に選択インデックスを設定
local tmpOpt, changedOpt = {}, {}
for i=1, #optionAll.Menu do
    local key = optionAll.Menu[i]
    -- UnJoin設定は二人プレイの時だけ表示
    if key ~= 'UnJoin' or W_PLAYER:NumJoined() >= 2 then
        option.Menu[#(option.Menu)+1] = optionAll.Menu[i]
        local val = W_OPTION:ReadPref(key)
        if option.OptionList[key].Type ~= 'None' then
            for j=1, #option.OptionList[key].List do
                if val == option.OptionList[key].List[j] and not tmpOpt[key] then
                    tmpOpt[key] = j
                end
            end
            if not tmpOpt[key] then
                tmpOpt[key] = option.OptionList[key].Default or 1 -- 取得できなかった場合のデフォルト値
            end
            changedOpt[key] = false -- trueの時のみ保存対象
        end
    end
end

local function Input(params)
    local function OverlayCancel(...)
        local wait = ...
        screen:lockinput(0.25)
        screen:RemoveInputCallback(Input)
        MESSAGEMAN:Broadcast('Off', {Wait = wait or nil})
    end
    local function LeftRight(move)
        local key = option.Menu[cursor]
        local opt = option.OptionList[key]
        if opt.Type ~= 'None' then
            local tmp = tmpOpt[key] + move
            tmpOpt[key] = math.min(math.max(tmpOpt[key] + move, 1), #opt.List)
            if tmp == tmpOpt[key] then
                MESSAGEMAN:Broadcast('PlaySound', {
                    Sound = 'Cursor',
                })
            end
            changedOpt[key] = true
            MESSAGEMAN:Broadcast('SetOptValue', tmpOpt)
        end
    end
    local function Default()
        local key = option.Menu[cursor]
        local opt = option.OptionList[key]
        if opt.Type ~= 'None' then
            MESSAGEMAN:Broadcast('PlaySound', {
                Sound = 'Cursor',
            })
            tmpOpt[key] = opt.Default
            changedOpt[key] = true
            MESSAGEMAN:Broadcast('SetOptValue', tmpOpt)
        end
    end
    if not screen then return end
    if params.type == 'InputEventType_FirstPress'
        or params.type == 'InputEventType_Repeat' then
        -- 変更せずに戻る
        if params.button == 'Back' then
            MESSAGEMAN:Broadcast('PlaySound', {
                Sound = 'Back',
            })
            setenv('WAIEI_PlayerUnjoined', false)
            OverlayCancel(0.2)
        -- 初期値
        elseif params.button == 'Select' then
            Default()
        -- カーソル移動
        elseif params.button == 'Up' then
            if cursor > 1 then
                cursor = cursor-1
            else
                cursor = #option.Menu
            end
            MESSAGEMAN:Broadcast('PlaySound', {
                Sound = 'Cursor',
            })
            MESSAGEMAN:Broadcast('Scroll', {Cursor = cursor})
        elseif params.button == 'Down' then
            if cursor < #option.Menu then
                cursor = cursor+1
            else
                cursor = 1
            end
            MESSAGEMAN:Broadcast('PlaySound', {
                Sound = 'Cursor',
            })
            MESSAGEMAN:Broadcast('Scroll', {Cursor = cursor})
        elseif params.button == 'Left' then
            LeftRight(-1)
        elseif params.button == 'Right' then
            LeftRight(1)
        -- 確定
        elseif params.button == 'Start' then
            local key = option.Menu[cursor]
            if key == 'UnJoin' then
                -- プレイヤーのUnJoin
                if W_PLAYER:NumJoined() >= 2 then
                    GAMESTATE:UnjoinPlayer(params.PlayerNumber)
                    MESSAGEMAN:Broadcast('PlayerUnjoined', {
                        Player = params.PlayerNumber,
                    })
                    setenv('WAIEI_PlayerUnjoined', true)
                    OverlayCancel()
                end
            elseif cursor == #option.Menu then
                MESSAGEMAN:Broadcast('PlaySound', {
                    Sound = 'Start',
                })
                for k,v in pairs(tmpOpt) do
                    if option.OptionList[k].Type ~= 'None' and changedOpt[k] then
                        local value = option.OptionList[k].List[v]
                        W_OPTION:SavePref(k, value)
                        if k == 'DifficultyColor' then
                            W_DIF:ColorType(value)
                        end
                        if k == 'DifficultyLabel' then
                            W_DIF:NameType(value)
                        end
                        if k == 'GradeType' then
                            W_SCORE:SetGradeType(value)
                        end
                        if k == 'HighScoreType' then
                            PREFSMAN:SetPreference('PercentageScoring', value)
                        end
                    end
                end
                THEME:ReloadMetrics()
                setenv('WAIEI_PlayerUnjoined', false)
                OverlayCancel(0.2)
            else
                cursor = #option.Menu
                MESSAGEMAN:Broadcast('Scroll', {Cursor = cursor})
                MESSAGEMAN:Broadcast('PlaySound', {
                    Sound = 'Start',
                })
            end
        end
    end
end

-- 選曲の方のタイマーが0になった場合強制的に閉じる
local timerChild = getenv('WAIEI_ScreenTimer')
local function update(self, delta)
    if timerChild then
        if tonumber(timerChild:GetSeconds()) <= 0 then
            timerChild:SetSeconds(0.01)
            timerChild:start()
            setenv('WAIEI_PlayerUnjoined', false)
            screen:lockinput(0.25)
            screen:RemoveInputCallback(Input)
            MESSAGEMAN:Broadcast('Off', {Wait = 0})
        end
    end
end

-- キー操作
return Def.ActorFrame({
    Def.ActorFrame({
        OnCommand = function(self)
            screen = SCREENMAN:GetTopScreen()
            screen:AddInputCallback(Input)
            self:SetUpdateFunction(update)
            MESSAGEMAN:Broadcast('SetOptValue', tmpOpt)
        end,
        OffMessageCommand = function(self, params)
            if params.Wait then
                self:sleep(params.Wait)
            end
            self:queuecommand('CancelOverlay')
        end,
        CancelOverlayMessageCommand = function(self)
            screen:Cancel()
        end,
        -- 音
        LoadActor(THEME:GetPathS('Common', 'value'))..{
            InitCommand = function(self) W:SE(self) end,
            PlaySoundMessageCommand = function(self, params)
                if params.Sound == 'Cursor' then
                    self:play()
                end
            end,
        },
        LoadActor(THEME:GetPathS('Common', 'start'))..{
            InitCommand = function(self) W:SE(self) end,
            PlaySoundMessageCommand = function(self, params)
                if params.Sound == 'Start' then
                    self:play()
                end
            end,
        },
        LoadActor(THEME:GetPathS('Common', 'Cancel'))..{
            InitCommand = function(self) W:SE(self) end,
            PlaySoundMessageCommand = function(self, params)
                if params.Sound == 'Back' then
                    self:play()
                end
            end,
        },
    }),
})
