local w, h = 240, 380
local profileMax = PROFILEMAN:GetNumLocalProfiles()

local function ActorTarget(pn)
    local actors = Def.ActorFrame({
        InitCommand = function(self)
            self:y(115)
        end,
        SelectProfileChildCommand = function(self, params)
            if params.Player ~= pn then return end
            self:visible(params.Profile ~= nil)
            if params.Profile then
                local id = PROFILEMAN:GetLocalProfileIDFromIndex(params.Index-1)
                local profileTarget = split('|', W_PLAYER:ProfilePref(id, 'TargetList', 1, ''))
                local pnSetting = W_PLAYER:ProfilePref(id, 'EnableTarget', 1, 'Off')
                if pnSetting ~= 'On' then self:visible(false) return end
                self:playcommand('SetProfileTarget', {
                    ProfileIndex = params.Index,
                    ProfileName = params.Profile:GetDisplayName(),
                    Target = profileTarget,
                })
            end
        end,
    })

    local function ActorTarget()
        local offsetY = 0
        local targets = Def.ActorFrame({
            DownYCommand = function(self, params)
                if params.Y ~= 0 and offsetY > 0 then return end
                offsetY = params.Y
                self:y(offsetY)
            end,
        })
        for i=1,3 do
            targets[#targets+1] = Def.ActorFrame({
                SetProfileTargetCommand = function(self, params)
                    local target = params.Target[i]
                    -- TargetScoreオプションが有効な場合はハイスコアが対象となるが、ターゲット2以降は関係無いのでreturn
                    if ((not target or target == '') and i > 1) then
                        self:visible(false)
                        self:playcommand('SetTargetName', {})
                        -- 3件無い場合は縦方向にずらす
                        self:GetParent():playcommand('DownY', {Y = (i == 2) and 40 or 20})
                        return
                    end
                    self:GetParent():playcommand('DownY', {Y = 0})
                    self:visible(true)
                    local isSystem
                    local systemName
                    local targetName
                    if (not target or target == '') then
                        isSystem = true
                        systemName = 'HighScore'
                        targetName = params.ProfileName
                    else
                        isSystem, systemName = W_TARGET:IsSystem(target)
                        targetName = W_TARGET:Name(target)
                    end
                    local avatarPath = W_TARGET:Avatar(target)
                    if isSystem and systemName == 'HighScore' then
                        avatarPath = W_PLAYER:IndexAvatar(params.ProfileIndex-1)
                    end
                    self:playcommand('SetTargetName', {Index = i, Name = targetName})
                    self:playcommand('ChangeAvatarFile', {
                        Id = 'Target'..i,
                        RandomNum = W_AVATAR:RandomNum(targetName),
                        File = avatarPath,
                    })
                end,
                W_AVATAR:Frame(W_PLAYER:PlayerAvatar(pn), 'Target'..i, {
                    Mini  = true,
                    Flip  = false,
                    Type  = 1,
                    Color = color('#666666ff'),
                    RandomNum = W_AVATAR:RandomNum(W_PLAYER:Name(pn)),
                })..{
                    OnCommand = function(self)
                        self:zoom(0.3)
                        self:xy(w/2 - 55, 5 + (i-1) * 40)
                    end,
                },
                LoadFont('Common Small Normal')..{
                    Text = (i == 1) and 'MAIN TARGET' or 'TARGET '..i,
                    InitCommand = function(self)
                        self:skewx(-0.2)
                        self:zoom(0.35)
                        self:horizalign(left)
                        self:xy(-w/2+40, -5 + (i-1) * 40)
                    end,
                    SetTargetNameCommand = function(self, params)
                        if params.Index ~= i then return end
                        self:visible(params.Name ~= nil)
                    end,
                },
                LoadFont('Common Small Bold')..{
                    InitCommand = function(self)
                        self:zoom(0.45)
                        self:horizalign(left)
                        self:xy(-w/2+40, 10 + (i-1) * 40)
                        self:maxwidth(100/0.45)
                    end,
                    SetTargetNameCommand = function(self, params)
                        if params.Index ~= i then return end
                        self:visible(params.Name ~= nil)
                        self:settext(params.Name or '')
                    end,
                },
            })
        end
        return targets
    end

    actors[#actors+1] = ActorTarget()

    actors[#actors+1] = LoadActor(W:Object('boxline'), 'Target', {
        Width = w - 40,
        Height = 135,
        Border = 1,
        Color = color('#ffffff80'),
    })..{
        InitCommand = function(self)
            self:y(45)
        end,
    }
    return actors
end

-- 特定の書式
local function TextTemplateActor(textList)
    return Def.ActorFrame({
        InitCommand = function(self)
            self:zoom(0.9)
        end,
        LoadFont('Common Small Normal')..{
            Name = 'Label',
            Text = textList[1],
            InitCommand = function(self)
                self:diffuse(Color('White'))
                self:zoom(0.35)
                self:horizalign(left)
                self:vertalign(top)
                self:xy(-w/2+10, 0)
            end,
        },
        LoadFont('Common Normal')..{
            Name = 'Value',
            Text = textList[2],
            InitCommand = function(self)
                self:diffuse(Color('White'))
                self:zoom(0.6)
                self:horizalign(right)
                self:vertalign(bottom)
                self:xy(w/2-40, 18)
            end,
        },
        LoadFont('Common Small Normal')..{
            Name = 'Unit',
            Text = textList[3],
            InitCommand = function(self)
                self:diffuse(Color('White'))
                self:zoom(0.35)
                self:horizalign(right)
                self:vertalign(bottom)
                self:xy(w/2-10, 18)
            end,
        },
    })
end

-- アイコン一覧
local function IconAvatar(pn, index)
    local profile = index > 0 and PROFILEMAN:GetLocalProfileFromIndex(index-1) or nil
    return Def.ActorFrame({
        InitCommand = function(self)
            if profileMax < (w/40) then
                self:x(-w/2 + 20 + index * 40)
            else
                self:x(-w/2 + 20 + (w-40) * index / profileMax)
            end
            self:y(-(h-40)/2-4)
            self:visible(false)
            self:draworder(190-index)
        end,
        JoinCommand = function(self, params)
            -- paramsが指定されている場合はプレイヤーチェック
            if params and params.Player ~= pn then return end
            self:finishtweening()
            self:diffusealpha(0)
            self:addx(W:IsP1(pn) and -50 or 50)
            self:sleep(0.02 * index)
            self:linear(0.2)
            self:addx(W:IsP1(pn) and 50 or -50)
            self:diffusealpha(1)
        end,
        BackMessageCommand = function(self, params)
            if params and params.Player ~= pn then return end
            self:finishtweening()
            self:sleep(0.02 * index)
            self:linear(0.2)
            self:addx(W:IsP1(pn) and -50 or 50)
            self:diffusealpha(0)
            self:sleep(0.1)
            self:addx(W:IsP1(pn) and 50 or -50)
        end,
        PlayerOKMessageCommand = function(self, params)
            self:playcommand('Back', params)
        end,
        PlayerNGMessageCommand = function(self, params)
            self:playcommand('Join', params)
        end,
        Name = index,
        SelectProfileChildCommand = function(self, params)
            if params.Player ~= pn then return end
            if params.Index == index then
                self:draworder(200)
            else
                self:draworder(190-index)
            end
            self:visible(true)
            self:GetParent():SortByDrawOrder()
        end,
        Def.Quad({
            InitCommand = function(self)
                self:zoom(40, 40)
                self:y(4)
            end,
            SelectProfileChildCommand = function(self, params)
                if params.Player ~= pn then return end
                local col = BoostColor(W_PLAYER:Color(pn), 0.25)
                self:diffuseshift()
                if params.Index == index then
                    self:effectcolor1(1, 1, 1, 1)
                    self:effectcolor2(col[1], col[2], col[3], 0.75)
                    
                else
                    self:effectcolor1(col[1], col[2], col[3], 0.75)
                    self:effectcolor2(col[1], col[2], col[3], 0.75)
                end
            end,
        }),
        W_AVATAR:Actor(W_PLAYER:IndexAvatar(index-1), 'Icon'..W:PNum(pn)..index, {
            Mini  = true,
            Flip  = pn == PLAYER_1,
            Type  = 1,
            Color = BoostColor(W_PLAYER:Color(pn), 0.5),
            RandomNum = (index == 0) and 0 or W_AVATAR:RandomNum(profile:GetDisplayName()),
        })..{
            InitCommand = function(self)
                self:zoom(0.25)
            end,
        },
        LoadFont('Common Small Normal')..{
            InitCommand = function(self)
                self:zoom(0.3)
                self:y(18)
                if index > 0 then
                    self:settext(W_STRING:CutWidth(profile:GetDisplayName(), 5))
                else
                    self:settext('Guest')
                end
            end,
        }
    })
end

-- プレイヤー
local function PlayerActor(pn)
    local function IconListActor()
        local IconList = Def.ActorFrame({Name='IconList'})
        for i=0, profileMax do
            IconList[#IconList+1] = IconAvatar(pn, i)
        end
        return IconList
    end
    -- JOIN時
    local JoinActor = Def.ActorFrame({
        Name = 'Join',
        OnCommand = function(self)
            if not W_PLAYER:IsEnabled(pn) then return end
            self:diffusealpha(0)
            self:sleep(0.2)
            self:decelerate(0.2)
            self:diffusealpha(1)
        end,
        IconListActor(),
        Def.ActorFrame({
            JoinCommand = function(self, params)
                -- paramsが指定されている場合はプレイヤーチェック
                if params and params.Player ~= pn then return end
                self:diffusealpha(0)
                self:addy(-50)
                self:sleep(0.1)
                self:linear(0.2)
                self:addy(50)
                self:diffusealpha(1)
            end,
            BackMessageCommand = function(self, params)
                -- paramsが指定されている場合はプレイヤーチェック
                if params and params.Player ~= pn then return end
                self:sleep(0.1)
                self:linear(0.2)
                self:diffusealpha(0)
                self:sleep(0.1)
            end,
            -- W_PLAYER:IndexAvatar(index-1)
            W_AVATAR:Frame(W_PLAYER:PlayerAvatar(pn), 'Player'..W:PNum(pn), {
                Mini  = true,
                Flip  = pn == PLAYER_1,
                Type  = 0,
                Color = BoostColor(W_PLAYER:Color(pn), 0.5),
            })..{
                InitCommand = function(self)
                    self:xy(W:IsP1(pn) and -w/2+45 or w/2-45, -h/2 + 85)
                    self:zoom(0.5)
                end,
                SelectProfileChildCommand = function(self, params)
                    if params.Player ~= pn then return end
                    self:playcommand('ChangeAvatarFile', {
                        Id = 'Player'..W:PNum(pn),
                        File = W_PLAYER:IndexAvatar(params.Index-1),
                        RandomNum = params.Profile and W_AVATAR:RandomNum(params.Profile:GetDisplayName()) or 0,
                    })
                end,
            },
            LoadFont('Common Normal')..{
                Text = 'Player'..(W:PNum(pn)),
                InitCommand = function(self)
                    self:xy(W:IsP1(pn) and -30 or 30, -h/2 + 72)
                    self:horizalign(W:IsP1(pn) and left or right)
                    self:zoom(0.6)
                    self:diffuse(BoostColor(W_PLAYER:Color(pn), 2.0))
                    self:strokecolor(Color('Outline'))
                end,
            },
            LoadFont('Common Normal')..{
                InitCommand = function(self)
                    self:xy(W:IsP1(pn) and -30 or 30, -h/2 + 95)
                    self:horizalign(W:IsP1(pn) and left or right)
                    self:zoom(0.8)
                    self:maxwidth((w/2+15)/0.8)
                    self:diffuse(1, 1, 1, 1)
                    self:strokecolor(Color('Outline'))
                end,
                SelectProfileChildCommand = function(self, params)
                    if params.Player ~= pn then return end
                    if params.Profile then
                        self:settext(params.Profile:GetDisplayName())
                    else
                        self:settext('Guest')
                    end
                end,
            },
        }),
        Def.ActorFrame({
            InitCommand = function(self)
                self:y(-h/2 + 135)
                self:diffuse(1, 1, 1, 0.8)
            end,
            JoinCommand = function(self, params)
                -- paramsが指定されている場合はプレイヤーチェック
                if params and params.Player ~= pn then return end
                self:diffusealpha(0)
                self:addx(W:IsP1(pn) and 50 or -50)
                self:sleep(0.1)
                self:linear(0.2)
                self:addx(W:IsP1(pn) and -50 or 50)
                self:diffusealpha(1)
            end,
            BackMessageCommand = function(self, params)
                -- paramsが指定されている場合はプレイヤーチェック
                if params and params.Player ~= pn then return end
                self:sleep(0.1)
                self:linear(0.2)
                self:addx(W:IsP1(pn) and 50 or -50)
                self:diffusealpha(0)
                self:sleep(0.1)
                self:addx(W:IsP1(pn) and -50 or 50)
            end,
            -- 本日の累計消費カロリー
            TextTemplateActor({
                THEME:GetString('Player', 'CaloriesBurnedToday'),
                '',
                'Kcal',
            })..{
                InitCommand = function(self)
                    self:y(0)
                    self:diffuse(1, 1, 1, 0.8)
                end,
                SelectProfileChildCommand = function(self, params)
                    if params.Player ~= pn then return end
                    local child = self:GetChild('Value')
                    if params.Profile then
                        child:settext(commify(string.format('%4.1f', params.Profile:GetCaloriesBurnedToday()), ','))
                    else
                        child:settext('-')
                    end
                end,
            },
            -- これまでの累計消費カロリー
            TextTemplateActor({
                THEME:GetString('Player', 'TotalCaloriesBurned'),
                '',
                'Kcal',
            })..{
                InitCommand = function(self)
                    self:y(30)
                    self:diffuse(1, 1, 1, 0.8)
                end,
                SelectProfileChildCommand = function(self, params)
                    if params.Player ~= pn then return end
                    local child = self:GetChild('Value')
                    if params.Profile then
                        child:settext(commify(string.format('%4.1f', params.Profile:GetTotalCaloriesBurned()), ','))
                    else
                        child:settext('-')
                    end
                end,
            },
            -- 累計プレイ曲数
            TextTemplateActor({
                THEME:GetString('Player', 'TotalSongsPlayed'),
                '',
                'Songs',
            })..{
                InitCommand = function(self)
                    self:y(60)
                    self:diffuse(1, 1, 1, 0.8)
                end,
                SelectProfileChildCommand = function(self, params)
                    if params.Player ~= pn then return end
                    local child = self:GetChild('Value')
                    if params.Profile then
                    child:settext(commify(string.format('%3d', params.Profile:GetTotalNumSongsPlayed()), ','))
                    else
                        child:settext('-')
                    end
                end,
            },
            ActorTarget(pn),
        }),
    })

    local UnjoinActor = Def.ActorFrame({
        Name = 'UnJoin',
        OnCommand = function(self)
            if W_PLAYER:IsEnabled(pn) then return end
            self:diffusealpha(0)
            self:sleep(0.2)
            self:decelerate(0.2)
            self:diffusealpha(1)
        end,
        Def.Quad({
            InitCommand = function(self)
                local pCol = BoostColor(W_PLAYER:Color(pn), 0.75)
                self:zoomto(w, h)
                self:diffuseshift()
                self:effectcolor1(pCol[1], pCol[2], pCol[3], 0.3)
                self:effectcolor2(0.5, 0.5, 0.5, 0.3)
            end,
        }),
        LoadFont('Common Normal')..{
            InitCommand = function(self)
                self:zoom(0.8)
                self:maxwidth((w-20)/0.8)
                self:strokecolor(Color('Outline'))
                self:settextf('Player%d &Start;で参加', W:PNum(pn))
                self:vertspacing(10)
            end,
        },
    })

    return Def.ActorFrame({
        OnCommand = function(self)
            self:diffusealpha(0)
            self:zoomx(2)
            self:zoomy(0)
            self:addy(200)
            self:decelerate(0.2)
            self:addy(-200)
            self:diffusealpha(1)
            self:zoomx(1)
            self:zoomy(1)
            self:decelerate(0.2)
            self:x(180 * (W:IsP1(pn) and -1 or 1))
        end,
        FinishMessageCommand = function(self)
            self:bouncebegin(0.4)
            self:addx((SCREEN_CENTER_X + 90) * (W:IsP1(pn) and -1 or 1))
        end,
        SelectProfileMessageCommand = function(self, params)
            if W_PLAYER:IsEnabled(pn) then
                -- 何故か一部childにメッセージが飛ばないので直接呼ぶ（要調査）
                for k,v in pairs(self:GetChild('Join'):GetChild('IconList'):GetChildren()) do
                    v:playcommand('SelectProfileChild', {
                        Player  = params.Player,
                        Index   = params.Index,
                        Profile = params.Index > 0 and PROFILEMAN:GetLocalProfileFromIndex(params.Index-1) or nil,
                    })
                end
                self:playcommand('SelectProfileChild', {
                    Player  = params.Player,
                    Index   = params.Index,
                    Profile = params.Index > 0 and PROFILEMAN:GetLocalProfileFromIndex(params.Index-1) or nil,
                })
            end
        end,
        FinishOrJoinMessageCommand = function(self, params)
            if params.Player ~= pn then return end
            if params.Join then
                self:playcommand('SelectProfile', {
                    Player  = params.Player,
                    Index   = 0,
                    Profile = nil,
                })
                self:GetParent():playcommand('Join', {Player = pn})
            end
        end,
        LoadActor(W:Object('boxshadow'), 'Background', {
            Length = 20,
            Color  = {0, 0, 0, 0.5},
            Center = {0.2, 0.2, 0.2, 0.75},
        })..{
            OnCommand = function(self, params)
                self:playcommand('UpdateBoxShadow', {
                    Id         = 'Background',
                    Width      = w,
                    Height     = h,
                })
            end,
        },
        JoinActor..{
            JoinCommand = function(self, params)
                -- paramsが指定されている場合はプレイヤーチェック
                if params and params.Player ~= pn then return end
                self:visible(W_PLAYER:IsEnabled(pn))
            end,
        },
        UnjoinActor..{
            InitCommand = function(self)
                self:diffusealpha(W_PLAYER:IsEnabled(pn) and 0 or 1)
            end,
            JoinCommand = function(self, params)
                -- paramsが指定されている場合はプレイヤーチェック
                if not W_PLAYER:IsEnabled(pn) then return end
                self:finishtweening()
                self:linear(0.2)
                self:diffusealpha(0)
            end,
            UnJoinMessageCommand = function(self, params)
                -- paramsが指定されている場合はプレイヤーチェック
                if params and params.Player ~= pn then return end
                self:finishtweening()
                self:linear(0.2)
                self:diffusealpha(1)
            end,
        },
    })
end

return Def.ActorFrame({
    OnCommand = function(self)
        self:playcommand('Join')
    end,
    UnJoinMessageCommand = function(self, params)
        self:playcommand('Back', params)
    end,
    FinishMessageCommand = function(self)
        self:playcommand('Back')
    end,
    PlayerActor(PLAYER_1),
    PlayerActor(PLAYER_2),
    Def.HelpDisplay({
        File = 'Common Small Normal',
        InitCommand=function(self)
            self:diffuse(Color('White'))
            self:zoom(0.4);
            self:xy(0, SCREEN_CENTER_Y-12)
            self:SetSecsBetweenSwitches(3.0)
            self:SetTipsColonSeparated(THEME:GetString('Screen', 'HelperTextDefault'))
        end,
        FinishMessageCommand = function(self)
            self:sleep(0.2)
            self:linear(0.3)
            self:addy(30)
        end,
    }),
})