local screen
local index = {PlayerNumber_P1 = -1, PlayerNumber_P2 = -1}
local cursor = {PlayerNumber_P1 = 0, PlayerNumber_P2 = 0}
local selected = {PlayerNumber_P1 = false, PlayerNumber_P2 = false}
local tryConnect = {PlayerNumber_P1 = false, PlayerNumber_P2 = false}
local lock = true

local profileMax = PROFILEMAN:GetNumLocalProfiles()

local function GetStartIndex(pn)
    local ret = -1
    local def = PREFSMAN:GetPreference('DefaultLocalProfileIDP'..W:PNum(pn))
    ret = (def ~= '') and PROFILEMAN:GetLocalProfileIndexFromID(def) + 1 or 0
    return ret
end

for pn in ivalues(PlayerNumber) do
    -- カーソル初期値はデフォルトPROFILE
    if W_PLAYER:IsEnabled(pn) then
        index[pn] = GetStartIndex(pn)
    end
    -- JOINしていないプレイヤーは選択済み扱い
    selected[pn] = not W_PLAYER:IsEnabled(pn)
    tryConnect[pn] = not W_PLAYER:IsEnabled(pn)
end

local function OKCheck(...)
    local pn = ...
    if pn then
        return selected[pn]
    end
    return selected[PLAYER_1] and selected[PLAYER_2]
end

-- プレイヤーのPROFILEを確定する
local function SetPlayerProfile(pn)
    if selected[pn] then return false end
    -- Finish
    selected[pn] = true
    if index[pn] == index[W:APn(pn)] and OKCheck(W:APn(pn)) then
        -- 同じPROFILEを選択した場合は強制ゲスト
        screen:SetProfileIndex(pn, -1)
        index[pn] = 0
    else
        if index[pn] > 0 then
            screen:SetProfileIndex(pn, index[pn])
        else
            screen:SetProfileIndex(pn, -1)
        end
    end
    -- ゲストユーザーの場合オプションリセット
    if index[pn] == 0 then
        W_OPTION:ResetGuest(pn)
    end
    return true
end


local function Input(params)
    if lock then return end
    local pn = params.PlayerNumber
    local function ScreenCancel()
        screen:RemoveInputCallback(Input)
        MESSAGEMAN:Broadcast('Back')
    end
    local function LeftRight(move, cur, pn)
        if profileMax < 1 then return end
        MESSAGEMAN:Broadcast('PlaySound', {
            Sound = 'Cursor',
        })
        if cur == 0 then
            if W_PLAYER:IsEnabled(pn) then
                -- PROFILE
                index[pn] = index[pn] + move
                if index[pn] < 0 then index[pn] = profileMax end
                if index[pn] > profileMax then index[pn] = 0  end
                screen:SetProfileIndex(pn, index[pn])
                MESSAGEMAN:Broadcast('SelectProfile', {
                    Player = pn,
                    Index  = index[pn],
                })
            end
        end
    end
    local function IsFirst(p)
        return p == 'InputEventType_FirstPress'
    end
    local function IsRepeat(p)
        return p == 'InputEventType_Repeat'
    end
    if not screen then return end
    if params.button == 'Start' then
        if IsFirst(params.type) then
            local join = false
            if W_PLAYER:IsEnabled(pn) then
                if SetPlayerProfile(pn) then
                    MESSAGEMAN:Broadcast('PlaySound', {
                        Sound = 'Start',
                    })
                end
                join = false
            else
                -- Join
                selected[pn] = false
                index[pn] = GetStartIndex(pn)
                screen:SetProfileIndex(pn, -1)
                screen:SetProfileIndex(pn, index[pn])
                join = true
            end
            MESSAGEMAN:Broadcast('FinishOrJoin', {
                Player = pn,
                Join   = join,
            })
            MESSAGEMAN:Broadcast('SelectProfile', {
                Player = pn,
                Index  = index[pn],
            })
        end
    end
    -- 決定キー以外は未ログインプレイヤーの操作を受け付けない
    if pn and not W_PLAYER:IsEnabled(pn) then return end
    if IsFirst(params.type) or IsRepeat(params.type) then
        -- 戻る
        if params.button == 'Back' and IsFirst(params.type) then
            MESSAGEMAN:Broadcast('PlaySound', {
                Sound = 'Back',
            })
            if selected[pn] then
                selected[pn] = false
                MESSAGEMAN:Broadcast('PlayerNG', {
                    Player = pn,
                })
            elseif index[W:APn(pn)] ~= -1 then
                -- 他プレイヤーが存在する場合はUnjoinする
                selected[pn] = true
                index[pn] = -1
                screen:SetProfileIndex(pn, -2)
                MESSAGEMAN:Broadcast('UnJoin', {
                    Player = pn,
                })
            else
                lock = true
                ScreenCancel()
            end
        -- 特殊メニュー（作るか未定）
        elseif params.button == 'Select' then
        -- カーソル移動
        elseif params.button == 'Up' then
            -- cursor[pn] = cursor[pn]-1
            -- if cursor < 0 then
            --     cursor = 0
            -- else
            --     MESSAGEMAN:Broadcast('Scroll', {Cursor = cursor})
            -- end
        elseif params.button == 'Down' then
            -- cursor[pn] = cursor[pn]+1
            -- if cursor >= #option.Menu then
            --     cursor = #option.Menu-1
            -- else
            --     MESSAGEMAN:Broadcast('Scroll', {Cursor = cursor})
            -- end
        elseif params.button == 'Left' then
            if selected[pn] then return end
            LeftRight(-1, cursor[pn], pn)
        elseif params.button == 'Right' then
            if selected[pn] then return end
            LeftRight(1, cursor[pn], pn)
        -- 確定
        end
    end
end

local function update(self, delta)
    if not screen or lock then return end
    local timerChild = screen:GetChild('Timer')
    if timerChild then
        if tonumber(timerChild:GetSeconds()) <= 0 then
            timerChild:SetSeconds(0.01)
            timerChild:pause()
            for pn in ivalues(PlayerNumber) do
                if W_PLAYER:IsEnabled(pn) then
                    SetPlayerProfile(pn)
                    MESSAGEMAN:Broadcast('FinishOrJoin', {
                        Player = pn,
                        Join   = false,
                    })
                    MESSAGEMAN:Broadcast('SelectProfile', {
                        Player = pn,
                        Index  = index[pn]+1,
                    })
                end
            end
            MESSAGEMAN:Broadcast('PlaySound', {
                Sound = 'Start',
            })
            lock = true
        end
    end
end

-- キー操作
return Def.ActorFrame({
    Def.ActorFrame({
        -- 初期遷移
        OnCommand = function(self)
            MESSAGEMAN:Broadcast('SelectProfile', {
                Player  = PLAYER_1,
                Index   = index[PLAYER_1],
                Profile = nil,
            })
            MESSAGEMAN:Broadcast('SelectProfile', {
                Player  = PLAYER_2,
                Index   = index[PLAYER_2],
                Profile = nil,
            })
            screen = SCREENMAN:GetTopScreen()
            screen:AddInputCallback(Input)
            self:SetUpdateFunction(update)
            self:sleep(0.5)
            self:queuecommand('Ready')
        end,
        ReadyCommand = function(self)
            lock = false
        end,
        -- 決定ボタン操作
        FinishOrJoinMessageCommand = function(self, params)
            if OKCheck() then
                -- 全プレイヤーがOKなら次の画面へ
                lock = true
                -- 終了エフェクト
                MESSAGEMAN:Broadcast('OffHeader')
                MESSAGEMAN:Broadcast('Finish')
                lock = true
                self:sleep(0.5)
                MESSAGEMAN:Broadcast('PlayerOK')
                self:queuecommand('FinishScreen')
            elseif not params.Join then
                -- そうでない場合は決定したプレイヤーの情報を送る
                MESSAGEMAN:Broadcast('PlayerOK', {
                    Player = selected[PLAYER_1] and PLAYER_1 or PLAYER_2,
                })
            end
        end,
        FinishScreenMessageCommand = function(self)
            screen:Finish()
            SCREENMAN:SetNewScreen(W_BRANCHE:Profile('Finish'))
        end,
        -- キャンセル処理
        BackMessageCommand = function(self)
            self:sleep(0.2)
            self:queuecommand('CancelScreen')
        end,
        CancelScreenMessageCommand = function(self)
            screen:Cancel()
        end,
        -- 音
        LoadActor(THEME:GetPathS('Common', 'value'))..{
            InitCommand = function(self) W:SE(self) end,
            PlaySoundMessageCommand = function(self, params)
                if params.Sound == 'Cursor' then
                    self:play()
                end
            end,
        },
        LoadActor(THEME:GetPathS('Common', 'start'))..{
            InitCommand = function(self) W:SE(self) end,
            PlaySoundMessageCommand = function(self, params)
                if params.Sound == 'Start' then
                    self:play()
                end
            end,
        },
        LoadActor(THEME:GetPathS('Common', 'Cancel'))..{
            InitCommand = function(self) W:SE(self) end,
            PlaySoundMessageCommand = function(self, params)
                if params.Sound == 'Back' then
                    self:play()
                end
            end,
        },
    }),
})