-- 歌詞表示エフェクトのテンプレート
-- Group.ini/luaのLyricTypeで意図的に定義しなくても利用可能にするための定義
--[[
    例）
    Group.ini
    #LyricType:Old;
    #LyricType:StepMania3|songA|songB:StepMania5|songC;

    Group.lua
    LyricType = {
        StepMania3 = {'oldSong'},
        LeftAndRight = {'songC'},
    },
    LyricType = {
        {
            Default = {'Duet', {
                Color1 = Color('White'),
                Color2 = Color('Black'),
            }},
            Rainbow = function(self, lyric, index)
                self:maxwidth(SCREEN_WIDTH - 20)
                self:rainbow()
                local parent = self:GetParent()
                parent:diffusealpha(0)
                parent:linear(0.1)
                parent:diffusealpha(0.9)
                parent:sleep(math.max(((index >= #lyric) and 9999 or lyric[index+1].Time - lyric[index].Time) - 0.3, 0.01))
                parent:linear(0.2)
                parent:diffusealpha(0)
            end,
        },
        Rainbow = {'songA', 'songC'},
    },
--]]

-- 通常表示
local function Default(self, lyric, index, ...)
    local attribute = ...
    self:maxwidth(SCREEN_WIDTH - 20)
    self:diffusealpha(0)
    self:linear(0.1)
    self:diffusealpha(0.9)
    self:sleep(math.min(math.max(((index >= #lyric) and 9999 or lyric[index+1].Time - lyric[index].Time) - 0.3, 0.01), attribute and attribute.SleepMax or 9999))
    self:linear(0.2)
    self:diffusealpha(0)
end

local data = {
    -- StepMania 3.9系
    -- 3秒で強制的に非表示
    -- キー名（大文字小文字は区別しない）：StepMania3
    stepmania3 = function(self, lyric, index, attribute)
        Default(self, lyric, index, {SleepMax = 2.7})
    end,
    -- StepMania3と同じ
    old = function(self, lyric, index, attribute)
        Default(self, lyric, index, {SleepMax = 2.7})
    end,
    legacy = function(self, lyric, index, attribute)
        Default(self, lyric, index, {SleepMax = 2.7})
    end,
    classic = function(self, lyric, index, attribute)
        Default(self, lyric, index, {SleepMax = 2.7})
    end,
    -- StepMania 5.0系（5.1も同じ）
    -- キー名（大文字小文字は区別しない）：StepMania5
    stepmania5 = function(self, lyric, index, attribute)
        Default(self, lyric, index)
    end,
    -- StepMania5と同じ
    default = function(self, lyric, index, attribute)
        Default(self, lyric, index)
    end,
    -- 左右に大きくゆったり表示
    -- キー名（大文字小文字は区別しない）：LeftAndRight
    leftandright = function(self, lyric, index, attribute)
        local offset = (8-math.min(mbstrlen(lyric[index].Text or ''), 8))*15
        self:strokecolor(color('0, 0, 0, 0'))
        self:shadowlength(2)
        self:zoom(2.5)
        self:maxwidth((SCREEN_WIDTH-40)/2.5)
        self:horizalign(((index%2) == 1) and left or right)
        self:x((((index%2) == 1) and (120 + offset) or SCREEN_RIGHT-(120 + offset)))
        self:y(SCREEN_CENTER_Y + (((index%2) == 1) and -30 or 70))
        self:diffusealpha(0)
        self:linear(0.1)
        self:diffusealpha(1.0)
        self:linear(math.max(((index >= #lyric) and 9999 or lyric[index+1].Time - lyric[index].Time) - 0.1, 0.01))
        self:addx(((index%2) == 1) and -100 or 100)
        self:linear(0.2)
        self:diffusealpha(0.5)
        self:linear(1)
        self:diffusealpha(0)
    end,
    -- 縦書きで画面いっぱいに埋める
    -- キー名（大文字小文字は区別しない）：VerticalFill
    verticalfill = function(self, lyric, index, attribute)
        -- 1文字ずつ改行
        local function f(t)
            local ret = {}
            local cnt = 0
            for c in string.gmatch (t, "[%z\1-\127\194-\244][\128-\191]*") do
                ret[#ret+1] = c
                cnt = cnt+1
            end
            return join('\n', ret), cnt
        end
        -- 空白行を除くと全体で何行になるのか計算
        local total, cur = 0, 0
        for i,l in pairs(lyric) do
            if l.Text ~= '' then
                total = total+1
            end
            if i == index then
                cur = total
            end
        end
        if total <= 0 then return end
        local text, count = f(string.gsub(lyric[index].Text, '\n', ''))
        self:vertspacing(0)
        self:strokecolor(attribute.StrokeColor or color('0, 0, 0, 0'))
        self:settext(text)
        self:zoomx((SCREEN_WIDTH)/total/21.8)
        self:zoomy((SCREEN_HEIGHT-20)/count/21.8)
        self:vertalign(top)
        local oneLine = (SCREEN_WIDTH) / (total+1)
        self:xy(SCREEN_WIDTH - (oneLine * cur), 10)
        self:cropright(1)
        self:linear(math.max(((index >= #lyric) and 9999 or lyric[index+1].Time - lyric[index].Time) - 0.5, 0.01))
        self:cropright(0)
    end,
    -- 中央でランダム回転表示
    -- キー名（大文字小文字は区別しない）：CenterRotation
    centerrotation = function(self, lyric, index, attribute)
        local zoom = math.random() + 1
        self:maxwidth(SCREEN_WIDTH - 20)
        self:Center()
        self:rotationz(math.random()*120-60)
        self:addx(math.random(-20, 20))
        self:addy(math.random(-20, 20))
        self:zoom(zoom + 1)
        self:diffusealpha(0)
        self:linear(0.1)
        self:zoom(zoom)
        self:diffusealpha(0.9)
        self:sleep(math.max(((index >= #lyric) and 9999 or lyric[index+1].Time - lyric[index].Time) - 0.3, 0.01))
        self:linear(0.2)
        self:diffusealpha(0.5)
        self:linear(9.8)
        self:diffusealpha(0)
    end,
    -- 右（左）下に表示
    -- キー名（大文字小文字は区別しない）：BottomSide
    bottomside = function(self, lyric, index, attribute)
        self:settext((lyric[index].Text ~= '') and '♫'..lyric[index].Text or '')
        self:strokecolor(color('0, 0, 0, 0.5'))
        --self:shadowlength(1)
        self:maxwidth((SCREEN_WIDTH-40)/1.5)
        self:horizalign((GAMESTATE:GetMasterPlayerNumber() == PLAYER_1) and right or left)
        self:vertalign(bottom)
        self:x((GAMESTATE:GetMasterPlayerNumber() == PLAYER_1) and SCREEN_RIGHT-20 or 20)
        self:y(SCREEN_CENTER_Y + 140)
        self:zoomx(3)
        self:zoomy(0)
        self:diffusealpha(0)
        self:linear(0.1)
        self:zoomx(1.5)
        self:zoomy(1.5)
        self:diffusealpha(1.0)
        self:sleep(math.max(((index >= #lyric) and 9999 or lyric[index+1].Time - lyric[index].Time) - 0.1, 0.01))
        self:linear(0.2)
        self:zoomx(3)
        self:zoomy(0)
        self:diffusealpha(0)
    end,
    -- デュエット
    -- テーブルで色の定義が可能（{'Duet', {Separator = 区切り文字(デフォルト\n), Color1 = 一人目, Color2 = 二人目}}）
    -- キー名（大文字小文字は区別しない）：Duet
    duet = function(self, lyric, index, attribute)
        local parent = self:GetParent()
        self:maxwidth(SCREEN_WIDTH - 20)
        self:ClearAttributes()
        local char = attribute and attribute.Separator or '\n'
        local part1s = string.match(lyric[index].Text, '^([^'..char..']+)'..char)
        if part1s then
            local part1len = mbstrlen(part1s)
            self:AddAttribute(0, {Length = part1len, Diffuse = attribute and attribute.Color1 or Color('Red')})
            self:AddAttribute(part1len, {Length = -1, Diffuse = attribute and attribute.Color2 or Color('Blue')})
        end
        parent:diffusealpha(0)
        parent:linear(0.1)
        parent:diffusealpha(0.9)
        parent:sleep(math.max(((index >= #lyric) and 9999 or lyric[index+1].Time - lyric[index].Time) - 0.3, 0.01))
        parent:linear(0.2)
        parent:diffusealpha(0)
    end,
}

return function(effect, self, lyric, index, ...)
    local attribute = ...
    local effectName = string.lower(effect)
    -- 存在しないキーを指定した場合はdefaultが呼び出される
    if data[effectName] then
        return data[effectName](self, lyric, index, attribute or {})
    else
        return data.default(self, lyric, index)
    end
end
